@@ -1588,6 +1588,9 @@ gdk_broadway_toplevel_set_property (GObject      *object,
     case LAST_PROP + GDK_TOPLEVEL_PROP_SHORTCUTS_INHIBITED:
       break;
 
+    case LAST_PROP + GDK_TOPLEVEL_PROP_GRAVITY:
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1638,6 +1641,16 @@ gdk_broadway_toplevel_get_property (GObject    *object,
       g_value_set_boolean (value, surface->shortcuts_inhibited);
       break;
 
+    case LAST_PROP + GDK_TOPLEVEL_PROP_CAPABILITIES:
+      g_value_set_flags (value, GDK_TOPLEVEL_CAPABILITIES_MAXIMIZE |
+                                GDK_TOPLEVEL_CAPABILITIES_FULLSCREEN |
+                                GDK_TOPLEVEL_CAPABILITIES_MINIMIZE);
+      break;
+
+    case LAST_PROP + GDK_TOPLEVEL_PROP_GRAVITY:
+      g_value_set_boolean (value, GDK_GRAVITY_NORTH_EAST);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1707,12 +1720,6 @@ gdk_broadway_toplevel_minimize (GdkToplevel *toplevel)
   return TRUE;
 }
 
-static gboolean
-gdk_broadway_toplevel_lower (GdkToplevel *toplevel)
-{
-  return FALSE;
-}
-
 static void
 gdk_broadway_toplevel_focus (GdkToplevel *toplevel,
                              guint32      timestamp)
@@ -1720,21 +1727,12 @@ gdk_broadway_toplevel_focus (GdkToplevel *toplevel,
   gdk_broadway_surface_focus (GDK_SURFACE (toplevel), timestamp);
 }
 
-static gboolean
-gdk_broadway_toplevel_show_window_menu (GdkToplevel *toplevel,
-                                        GdkEvent    *event)
-{
-  return FALSE;
-}
-
 static void
 gdk_broadway_toplevel_iface_init (GdkToplevelInterface *iface)
 {
   iface->present = gdk_broadway_toplevel_present;
   iface->minimize = gdk_broadway_toplevel_minimize;
-  iface->lower = gdk_broadway_toplevel_lower;
   iface->focus = gdk_broadway_toplevel_focus;
-  iface->show_window_menu = gdk_broadway_toplevel_show_window_menu;
   iface->begin_resize = gdk_broadway_toplevel_begin_resize;
   iface->begin_move = gdk_broadway_toplevel_begin_move;
 }
