@@ -18,6 +18,7 @@
 
 #include "config.h"
 
+#define GDK_COLOR_STATE_IMPL
 #include "gdkcolorstateprivate.h"
 
 #include <math.h>
@@ -29,8 +30,7 @@
 /**
  * GdkColorState:
  *
- * A `GdkColorState` object provides the information to interpret
- * colors and pixels in a variety of ways.
+ * Provides information to interpret colors and pixels in a variety of ways.
  *
  * They are also known as
  * [*color spaces*](https://en.wikipedia.org/wiki/Color_space).
@@ -229,6 +229,27 @@ gboolean
   return _gdk_color_state_equal (self, other);
 }
 
+/**
+ * gdk_color_state_equivalent:
+ * @self: a `GdkColorState`
+ * @other: another `GdkColorStatee`
+ *
+ * Compares two `GdkColorStates` for equivalence.
+ *
+ * Two objects that represent the same color state should be equivalent,
+ * even though they may not be equal in the sense of [method@Gdk.ColorState.equal].
+ *
+ * Returns: %TRUE if the two color states are equivalent
+ *
+ * Since: 4.20
+ */
+gboolean
+(gdk_color_state_equivalent) (GdkColorState *self,
+                              GdkColorState *other)
+{
+  return _gdk_color_state_equivalent (self, other);
+}
+
 /**
  * gdk_color_state_create_cicp_params:
  * @self: a `GdkColorState`
@@ -419,7 +440,7 @@ TRANSFORM_PAIR (rec2100_linear_to_oklch, rec2100_linear_to_oklab, oklab_to_oklch
 TRANSFORM_PAIR (oklch_to_srgb,           oklch_to_oklab,          oklab_to_srgb)
 TRANSFORM_PAIR (oklch_to_srgb_linear,    oklch_to_oklab,          oklab_to_srgb_linear)
 TRANSFORM_PAIR (oklch_to_rec2100_pq,     oklch_to_oklab,          oklab_to_rec2100_pq)
-TRANSFORM_PAIR (oklch_to_rec2100_linear, oklch_to_oklab,          oklab_to_rec2100_pq)
+TRANSFORM_PAIR (oklch_to_rec2100_linear, oklch_to_oklab,          oklab_to_rec2100_linear)
 
 /* }}} */
 /* {{{ Default implementation */
@@ -481,68 +502,6 @@ gdk_default_color_state_get_cicp (GdkColorState *color_state)
   return &self->cicp;
 }
 
-static gboolean
-gdk_color_state_check_inf_nan (const float src[4],
-                               float       dest[4])
-{
-  if (isnan (src[0]) ||
-      isnan (src[1]) ||
-      isnan (src[2]) ||
-      isnan (src[3]))
-    {
-      dest = (float[4]) { 1.0, 0.0, 0.8, 1.0 };
-      return TRUE;
-    }
-  if (isinf (src[0]) ||
-      isinf (src[1]) ||
-      isinf (src[2]) ||
-      isinf (src[3]))
-    {
-      dest = (float[4]) { 0.0, 0.8, 1.0, 1.0 };
-      return TRUE;
-    }
-
-  return FALSE;
-}
-
-static void
-gdk_color_state_clamp_0_1 (GdkColorState *self,
-                           const float    src[4],
-                           float          dest[4])
-{
-  if (gdk_color_state_check_inf_nan (src, dest))
-    return;
-
-  dest[0] = CLAMP (src[0], 0.0f, 1.0f);
-  dest[1] = CLAMP (src[1], 0.0f, 1.0f);
-  dest[2] = CLAMP (src[2], 0.0f, 1.0f);
-  dest[3] = CLAMP (src[3], 0.0f, 1.0f);
-}
-
-static void
-gdk_color_state_clamp_unbounded (GdkColorState *self,
-                                 const float    src[4],
-                                 float          dest[4])
-{
-  if (gdk_color_state_check_inf_nan (src, dest))
-    return;
-
-  dest[0] = src[0];
-  dest[1] = src[1];
-  dest[2] = src[2];
-  dest[3] = CLAMP (src[3], 0.0f, 1.0f);
-}
-
-static void
-gdk_default_color_state_clamp (GdkColorState *color_state,
-                               const float    in[4],
-                               float          out[4])
-{
-  GdkDefaultColorState *self = (GdkDefaultColorState *) color_state;
-
-  self->clamp (color_state, in, out);
-}
-
 /* }}} */
 
 static const
@@ -554,7 +513,6 @@ GdkColorStateClass GDK_DEFAULT_COLOR_STATE_CLASS = {
   .get_convert_to = gdk_default_color_state_get_convert_to,
   .get_convert_from = gdk_default_color_state_get_convert_from,
   .get_cicp = gdk_default_color_state_get_cicp,
-  .clamp = gdk_default_color_state_clamp,
 };
 
 GdkDefaultColorState gdk_default_color_states[] = {
@@ -563,6 +521,7 @@ GdkDefaultColorState gdk_default_color_states[] = {
       .klass = &GDK_DEFAULT_COLOR_STATE_CLASS,
       .ref_count = 0,
       .depth = GDK_MEMORY_U8_SRGB,
+      .hue_channel = GDK_COLOR_CHANNEL_ALPHA,
       .rendering_color_state = GDK_COLOR_STATE_SRGB,
       .rendering_color_state_linear = GDK_COLOR_STATE_SRGB_LINEAR,
     },
@@ -573,7 +532,6 @@ GdkDefaultColorState gdk_default_color_states[] = {
       [GDK_COLOR_STATE_ID_REC2100_PQ] = gdk_convert_srgb_to_rec2100_pq,
       [GDK_COLOR_STATE_ID_REC2100_LINEAR] = gdk_convert_srgb_to_rec2100_linear,
     },
-    .clamp = gdk_color_state_clamp_0_1,
     .cicp = { 1, 13, 0, 1 },
   },
   [GDK_COLOR_STATE_ID_SRGB_LINEAR] = {
@@ -581,6 +539,7 @@ GdkDefaultColorState gdk_default_color_states[] = {
       .klass = &GDK_DEFAULT_COLOR_STATE_CLASS,
       .ref_count = 0,
       .depth = GDK_MEMORY_U8,
+      .hue_channel = GDK_COLOR_CHANNEL_ALPHA,
       .rendering_color_state = GDK_COLOR_STATE_SRGB_LINEAR,
       .rendering_color_state_linear = GDK_COLOR_STATE_SRGB_LINEAR,
     },
@@ -591,7 +550,6 @@ GdkDefaultColorState gdk_default_color_states[] = {
       [GDK_COLOR_STATE_ID_REC2100_PQ] = gdk_convert_srgb_linear_to_rec2100_pq,
       [GDK_COLOR_STATE_ID_REC2100_LINEAR] = gdk_convert_srgb_linear_to_rec2100_linear,
     },
-    .clamp = gdk_color_state_clamp_0_1,
     .cicp = { 1, 8, 0, 1 },
   },
   [GDK_COLOR_STATE_ID_REC2100_PQ] = {
@@ -599,6 +557,7 @@ GdkDefaultColorState gdk_default_color_states[] = {
       .klass = &GDK_DEFAULT_COLOR_STATE_CLASS,
       .ref_count = 0,
       .depth = GDK_MEMORY_FLOAT16,
+      .hue_channel = GDK_COLOR_CHANNEL_ALPHA,
       .rendering_color_state = GDK_COLOR_STATE_REC2100_PQ,
       .rendering_color_state_linear = GDK_COLOR_STATE_REC2100_LINEAR,
     },
@@ -609,7 +568,6 @@ GdkDefaultColorState gdk_default_color_states[] = {
       [GDK_COLOR_STATE_ID_SRGB_LINEAR] = gdk_convert_rec2100_pq_to_srgb_linear,
       [GDK_COLOR_STATE_ID_REC2100_LINEAR] = gdk_convert_rec2100_pq_to_rec2100_linear,
     },
-    .clamp = gdk_color_state_clamp_0_1,
     .cicp = { 9, 16, 0, 1 },
   },
   [GDK_COLOR_STATE_ID_REC2100_LINEAR] = {
@@ -617,6 +575,7 @@ GdkDefaultColorState gdk_default_color_states[] = {
       .klass = &GDK_DEFAULT_COLOR_STATE_CLASS,
       .ref_count = 0,
       .depth = GDK_MEMORY_FLOAT16,
+      .hue_channel = GDK_COLOR_CHANNEL_ALPHA,
       .rendering_color_state = GDK_COLOR_STATE_REC2100_LINEAR,
       .rendering_color_state_linear = GDK_COLOR_STATE_REC2100_LINEAR,
     },
@@ -627,11 +586,12 @@ GdkDefaultColorState gdk_default_color_states[] = {
       [GDK_COLOR_STATE_ID_SRGB_LINEAR] = gdk_convert_rec2100_linear_to_srgb_linear,
       [GDK_COLOR_STATE_ID_REC2100_PQ] = gdk_convert_rec2100_linear_to_rec2100_pq,
     },
-    .clamp = gdk_color_state_clamp_unbounded,
     .cicp = { 9, 8, 0, 1 },
   },
 };
 
+G_STATIC_ASSERT (G_N_ELEMENTS (gdk_default_color_states) == GDK_COLOR_STATE_N_IDS);
+
 static gboolean
 gdk_builtin_color_state_equal (GdkColorState *self,
                                GdkColorState *other)
@@ -647,6 +607,30 @@ gdk_builtin_color_state_get_name (GdkColorState *color_state)
   return self->name;
 }
 
+/*<private>
+ * gdk_color_state_get_hue_channel:
+ * @self: the color state
+ * @out_channel: (out caller-allocates): the channel
+ *
+ * Checks if the color state has a hue channel and returns it if
+ * there is one.
+ * 
+ * Unlike other channels, hue channels are periodic, so they may
+ * need different treatment in various places.
+ *
+ * Returns: true if the color state has a hue channel
+ **/
+gboolean
+gdk_color_state_get_hue_channel (GdkColorState   *self,
+                                 GdkColorChannel *out_channel)
+{
+  if (self->hue_channel == GDK_COLOR_CHANNEL_ALPHA)
+    return FALSE;
+
+  *out_channel = self->hue_channel;
+  return TRUE;
+}
+
 static GdkColorState *
 gdk_builtin_color_state_get_no_srgb_tf (GdkColorState *color_state)
 {
@@ -659,7 +643,10 @@ gdk_builtin_color_state_get_convert_to (GdkColorState  *color_state,
 {
   GdkBuiltinColorState *self = (GdkBuiltinColorState *) color_state;
 
-  return self->convert_to[GDK_DEFAULT_COLOR_STATE_ID (target)];
+  if (GDK_IS_DEFAULT_COLOR_STATE (target))
+    return self->convert_to[GDK_DEFAULT_COLOR_STATE_ID (target)];
+
+  return NULL;
 }
 
 static GdkFloatColorConvert
@@ -668,7 +655,10 @@ gdk_builtin_color_state_get_convert_from (GdkColorState  *color_state,
 {
   GdkBuiltinColorState *self = (GdkBuiltinColorState *) color_state;
 
-  return self->convert_from[GDK_DEFAULT_COLOR_STATE_ID (source)];
+  if (GDK_IS_DEFAULT_COLOR_STATE (source))
+    return self->convert_from[GDK_DEFAULT_COLOR_STATE_ID (source)];
+
+  return NULL;
 }
 
 static const GdkCicp *
@@ -688,7 +678,6 @@ GdkColorStateClass GDK_BUILTIN_COLOR_STATE_CLASS = {
   .get_convert_to = gdk_builtin_color_state_get_convert_to,
   .get_convert_from = gdk_builtin_color_state_get_convert_from,
   .get_cicp = gdk_builtin_color_state_get_cicp,
-  .clamp = gdk_color_state_clamp_unbounded,
 };
 
 GdkBuiltinColorState gdk_builtin_color_states[] = {
@@ -697,6 +686,7 @@ GdkBuiltinColorState gdk_builtin_color_states[] = {
       .klass = &GDK_BUILTIN_COLOR_STATE_CLASS,
       .ref_count = 0,
       .depth = GDK_MEMORY_FLOAT16,
+      .hue_channel = GDK_COLOR_CHANNEL_ALPHA,
       .rendering_color_state = GDK_COLOR_STATE_SRGB,
     },
     .name = "oklab",
@@ -718,6 +708,7 @@ GdkBuiltinColorState gdk_builtin_color_states[] = {
       .klass = &GDK_BUILTIN_COLOR_STATE_CLASS,
       .ref_count = 0,
       .depth = GDK_MEMORY_FLOAT16,
+      .hue_channel = GDK_COLOR_CHANNEL_BLUE,
       .rendering_color_state = GDK_COLOR_STATE_SRGB,
     },
     .name = "oklch",
@@ -756,23 +747,157 @@ struct _GdkCicpColorState
   float from_srgb[9];
   float from_rec2020[9];
 
+  const float *from_yuv;
+  const float *to_yuv;
+
   GdkCicp cicp;
 };
 
 /* {{{ Conversion functions */
 
-#define cicp ((GdkCicpColorState *)self)
+#define TRANSFORM_FROM_CICP(name, matrix, oetf) \
+static void \
+name (GdkColorState  *color_state, \
+      float         (*values)[4], \
+      gsize           n_values) \
+{ \
+  GdkCicpColorState *self = (GdkCicpColorState *) color_state; \
+\
+  transform_from_cicp (self, oetf, self->matrix, values, n_values); \
+}
 
-TRANSFORM(cicp_to_srgb,             cicp->eotf,  cicp->to_srgb,      NONE, IDENTITY, srgb_oetf)
-TRANSFORM(cicp_to_srgb_linear,      cicp->eotf,  cicp->to_srgb,      NONE, IDENTITY, NONE)
-TRANSFORM(cicp_to_rec2100_pq,       cicp->eotf,  cicp->to_rec2020,   NONE, IDENTITY, pq_oetf)
-TRANSFORM(cicp_to_rec2100_linear,   cicp->eotf,  cicp->to_rec2020,   NONE, IDENTITY, NONE)
-TRANSFORM(cicp_from_srgb,           srgb_eotf,   cicp->from_srgb,    NONE, IDENTITY, cicp->oetf)
-TRANSFORM(cicp_from_srgb_linear,    NONE,        cicp->from_srgb,    NONE, IDENTITY, cicp->oetf)
-TRANSFORM(cicp_from_rec2100_pq,     pq_eotf,     cicp->from_rec2020, NONE, IDENTITY, cicp->oetf)
-TRANSFORM(cicp_from_rec2100_linear, NONE,        cicp->from_rec2020, NONE, IDENTITY, cicp->oetf)
+static void
+transform_from_cicp (GdkCicpColorState *self,
+                     GdkTransferFunc    oetf,
+                     float              matrix[9],
+                     float            (*values)[4],
+                     gsize              n_values)
+{
+  for (gsize i = 0; i < n_values; i++)
+    {
+      if (self->cicp.range == GDK_CICP_RANGE_NARROW)
+        {
+          values[i][0] = CLAMP ((values[i][0] - 16.0/255.0) * 255.0 / 224.0, 0.0, 1.0);
+          values[i][1] = CLAMP ((values[i][1] - 16.0/255.0) * 255.0 / 219.0, 0.0, 1.0);
+          values[i][2] = CLAMP ((values[i][2] - 16.0/255.0) * 255.0 / 224.0, 0.0, 1.0);
+        }
+      if (self->from_yuv)
+        {
+          float res[3];
+          values[i][0] -= 0.5;
+          values[i][2] -= 0.5;
+          res[0] = self->from_yuv[0] * values[i][0] + self->from_yuv[1] * values[i][1] + self->from_yuv[2] * values[i][2];
+          res[1] = self->from_yuv[3] * values[i][0] + self->from_yuv[4] * values[i][1] + self->from_yuv[5] * values[i][2];
+          res[2] = self->from_yuv[6] * values[i][0] + self->from_yuv[7] * values[i][1] + self->from_yuv[8] * values[i][2];
+          values[i][0] = res[0];
+          values[i][1] = res[1];
+          values[i][2] = res[2];
+        }
+      if (self->eotf != NONE)
+        {
+          values[i][0] = self->eotf (values[i][0]);
+          values[i][1] = self->eotf (values[i][1]);
+          values[i][2] = self->eotf (values[i][2]);
+        }
+      if (matrix != IDENTITY)
+        {
+          float res[3];
+          res[0] = matrix[0] * values[i][0] + matrix[1] * values[i][1] + matrix[2] * values[i][2];
+          res[1] = matrix[3] * values[i][0] + matrix[4] * values[i][1] + matrix[5] * values[i][2];
+          res[2] = matrix[6] * values[i][0] + matrix[7] * values[i][1] + matrix[8] * values[i][2];
+          values[i][0] = res[0];
+          values[i][1] = res[1];
+          values[i][2] = res[2];
+        }
+      if (oetf != NONE)
+        {
+          values[i][0] = oetf (values[i][0]);
+          values[i][1] = oetf (values[i][1]);
+          values[i][2] = oetf (values[i][2]);
+        }
+    }
+}
 
-#undef cicp
+#define TRANSFORM_TO_CICP(name, eotf, matrix) \
+static void \
+name (GdkColorState  *color_state, \
+      float         (*values)[4], \
+      gsize           n_values) \
+{ \
+  GdkCicpColorState *self = (GdkCicpColorState *) color_state; \
+\
+  transform_to_cicp (self, eotf, self->matrix, values, n_values); \
+}
+
+static void
+transform_to_cicp (GdkCicpColorState *self,
+                   GdkTransferFunc    eotf,
+                   float              matrix[9],
+                   float            (*values)[4],
+                   gsize              n_values)
+{
+  for (gsize i = 0; i < n_values; i++)
+    {
+      if (eotf != NONE)
+        {
+          values[i][0] = eotf (values[i][0]);
+          values[i][1] = eotf (values[i][1]);
+          values[i][2] = eotf (values[i][2]);
+        }
+      if (matrix != IDENTITY)
+        {
+          float res[3];
+          res[0] = matrix[0] * values[i][0] + matrix[1] * values[i][1] + matrix[2] * values[i][2];
+          res[1] = matrix[3] * values[i][0] + matrix[4] * values[i][1] + matrix[5] * values[i][2];
+          res[2] = matrix[6] * values[i][0] + matrix[7] * values[i][1] + matrix[8] * values[i][2];
+          values[i][0] = res[0];
+          values[i][1] = res[1];
+          values[i][2] = res[2];
+        }
+      else
+        {
+          float tmp = values[i][2];
+          values[i][2] = values[i][0];
+          values[i][0] = values[i][1];
+          values[i][1] = tmp;
+        }
+      if (self->oetf != NONE)
+        {
+          values[i][0] = self->oetf (values[i][0]);
+          values[i][1] = self->oetf (values[i][1]);
+          values[i][2] = self->oetf (values[i][2]);
+        }
+      if (self->to_yuv)
+        {
+          float res[3];
+          res[0] = self->to_yuv[0] * values[i][0] + self->to_yuv[1] * values[i][1] + self->to_yuv[2] * values[i][2];
+          res[1] = self->to_yuv[3] * values[i][0] + self->to_yuv[4] * values[i][1] + self->to_yuv[5] * values[i][2];
+          res[2] = self->to_yuv[6] * values[i][0] + self->to_yuv[7] * values[i][1] + self->to_yuv[8] * values[i][2];
+          values[i][0] = res[0] + 0.5;
+          values[i][1] = res[1];
+          values[i][2] = res[2] + 0.5;
+        }
+      values[i][0] = CLAMP (values[i][0], 0.0, 1.0);
+      values[i][1] = CLAMP (values[i][1], 0.0, 1.0);
+      values[i][2] = CLAMP (values[i][2], 0.0, 1.0);
+      if (self->cicp.range == GDK_CICP_RANGE_NARROW)
+        {
+          values[i][0] = values[i][0] * 224.0 / 255.0 + 16.0 / 255.0;
+          values[i][1] = values[i][1] * 219.0 / 255.0 + 16.0 / 255.0;
+          values[i][2] = values[i][2] * 224.0 / 255.0 + 16.0 / 255.0;
+        }
+    }
+}
+
+TRANSFORM_FROM_CICP(gdk_convert_cicp_to_srgb,           to_srgb,      srgb_oetf)
+TRANSFORM_FROM_CICP(gdk_convert_cicp_to_srgb_linear,    to_srgb,      NONE)
+TRANSFORM_FROM_CICP(gdk_convert_cicp_to_rec2100_pq,     to_rec2020,   pq_oetf)
+TRANSFORM_FROM_CICP(gdk_convert_cicp_to_rec2100_linear, to_rec2020,   NONE)
+
+TRANSFORM_TO_CICP(gdk_convert_cicp_from_srgb,           srgb_eotf,   from_srgb)
+TRANSFORM_TO_CICP(gdk_convert_cicp_from_srgb_linear,    NONE,        from_srgb)
+TRANSFORM_TO_CICP(gdk_convert_cicp_from_rec2100_pq,     pq_eotf,     from_rec2020)
+TRANSFORM_TO_CICP(gdk_convert_cicp_from_rec2100_linear, NONE,        from_rec2020)
 
 /* }}} */
 /* {{{ Vfuncs */
@@ -887,7 +1012,6 @@ GdkColorStateClass GDK_CICP_COLOR_STATE_CLASS = {
   .get_convert_to = gdk_cicp_color_state_get_convert_to,
   .get_convert_from = gdk_cicp_color_state_get_convert_from,
   .get_cicp = gdk_cicp_color_state_get_cicp,
-  .clamp = gdk_color_state_clamp_0_1,
 };
 
 static inline float *
@@ -914,14 +1038,8 @@ gdk_color_state_new_for_cicp (const GdkCicp  *cicp,
   GdkTransferFunc oetf;
   gconstpointer to_xyz;
   gconstpointer from_xyz;
-
-  if (cicp->range == GDK_CICP_RANGE_NARROW || cicp->matrix_coefficients != 0)
-    {
-      g_set_error (error,
-                   G_IO_ERROR, G_IO_ERROR_FAILED,
-                   _("cicp: Narrow range or YUV not supported"));
-      return NULL;
-    }
+  gconstpointer to_yuv = NULL;
+  gconstpointer from_yuv = NULL;
 
   if (cicp->color_primaries == 2 ||
       cicp->transfer_function == 2 ||
@@ -935,7 +1053,7 @@ gdk_color_state_new_for_cicp (const GdkCicp  *cicp,
 
   for (guint i = 0; i < GDK_COLOR_STATE_N_IDS; i++)
     {
-      if (gdk_cicp_equivalent (cicp, &gdk_default_color_states[i].cicp))
+      if (gdk_cicp_equal (cicp, &gdk_default_color_states[i].cicp))
         return (GdkColorState *) &gdk_default_color_states[i];
     }
 
@@ -991,6 +1109,7 @@ gdk_color_state_new_for_cicp (const GdkCicp  *cicp,
       from_xyz = xyz_to_pal;
       break;
     case 6:
+    case 7:
       to_xyz = ntsc_to_xyz;
       from_xyz = xyz_to_ntsc;
       break;
@@ -1010,6 +1129,34 @@ gdk_color_state_new_for_cicp (const GdkCicp  *cicp,
       return NULL;
     }
 
+  switch (cicp->matrix_coefficients)
+    {
+    case 0:
+      to_yuv = IDENTITY;
+      from_yuv = IDENTITY;
+      break;
+    case 1:
+      to_yuv = rgb_to_bt709;
+      from_yuv = bt709_to_rgb;
+      break;
+    case 5:
+    case 6:
+      to_yuv = rgb_to_bt601;
+      from_yuv = bt601_to_rgb;
+      break;
+    case 9:
+      to_yuv = rgb_to_bt2020;
+      from_yuv = bt2020_to_rgb;
+      break;
+    default:
+      g_set_error (error,
+                   G_IO_ERROR, G_IO_ERROR_FAILED,
+                   _("cicp: Matrix coefficients %u, %s not supported"),
+                   cicp->matrix_coefficients,
+                   cicp->range == GDK_CICP_RANGE_NARROW ? "narrow" : "full");
+      return NULL;
+    }
+
   self = g_new0 (GdkCicpColorState, 1);
 
   self->parent.klass = &GDK_CICP_COLOR_STATE_CLASS;
@@ -1020,9 +1167,13 @@ gdk_color_state_new_for_cicp (const GdkCicp  *cicp,
   self->parent.rendering_color_state_linear = GDK_COLOR_STATE_REC2100_LINEAR;
 
   self->parent.depth = GDK_MEMORY_FLOAT16;
+  self->parent.hue_channel = GDK_COLOR_CHANNEL_ALPHA;
 
   memcpy (&self->cicp, cicp, sizeof (GdkCicp));
 
+  self->to_yuv = to_yuv;
+  self->from_yuv = from_yuv;
+
   self->eotf = eotf;
   self->oetf = oetf;
 
@@ -1092,22 +1243,22 @@ gdk_color_state_get_no_srgb_tf (GdkColorState *self)
   return self->klass->get_no_srgb_tf (self);
 }
 
-/*< private >
- * gdk_color_state_clamp:
- * @self: a `GdkColorState`
- * @src: the values to clamp
- * @dest: (out): location to store the result, may be identical to
- *   the src argument
- *
- * Clamps the values to be within the allowed ranges for the given
- * color state.
- */
-void
-gdk_color_state_clamp (GdkColorState *self,
-                       const float    src[4],
-                       float          dest[4])
+GdkColorState *
+gdk_color_state_yuv (void)
 {
-  self->klass->clamp (self, src, dest);
+  static GdkColorState *yuv_cs;
+
+  if (g_once_init_enter_pointer (&yuv_cs))
+    {
+      GdkColorState *cs;
+
+      cs = gdk_color_state_new_for_cicp (&(GdkCicp) { 1, 13, 5, 0 }, NULL);
+      g_assert (cs);
+
+      g_once_init_leave_pointer (&yuv_cs, cs);
+    }
+
+  return yuv_cs;
 }
 
 /* }}} */
