@@ -31,6 +31,9 @@
 
 #ifdef GDK_RENDERING_VULKAN
 #include <vulkan/vulkan.h>
+#ifdef GDK_WINDOWING_WIN32
+#include <vulkan/vulkan_win32.h>
+#endif
 #endif
 
 G_BEGIN_DECLS
@@ -43,15 +46,21 @@ G_BEGIN_DECLS
 typedef struct _GdkDisplayClass GdkDisplayClass;
 
 typedef enum {
-  GDK_VULKAN_FEATURE_DMABUF                     = 1 << 0,
-  GDK_VULKAN_FEATURE_YCBCR                      = 1 << 1,
-  GDK_VULKAN_FEATURE_SEMAPHORE_EXPORT           = 1 << 2,
-  GDK_VULKAN_FEATURE_SEMAPHORE_IMPORT           = 1 << 3,
-  GDK_VULKAN_FEATURE_INCREMENTAL_PRESENT        = 1 << 4,
-  GDK_VULKAN_FEATURE_SWAPCHAIN_MAINTENANCE      = 1 << 5,
+  GDK_VULKAN_FEATURE_DUAL_SOURCE_BLEND          = 1 << 0,
+  GDK_VULKAN_FEATURE_PROFILE                    = 1 << 1,
+  GDK_VULKAN_FEATURE_DMABUF                     = 1 << 2,
+  GDK_VULKAN_FEATURE_WIN32                      = 1 << 3,
+  GDK_VULKAN_FEATURE_YCBCR                      = 1 << 4,
+  GDK_VULKAN_FEATURE_TIMELINE_SEMAPHORE         = 1 << 5,
+  GDK_VULKAN_FEATURE_SEMAPHORE_EXPORT           = 1 << 6,
+  GDK_VULKAN_FEATURE_SEMAPHORE_IMPORT           = 1 << 7,
+  GDK_VULKAN_FEATURE_WIN32_SEMAPHORE            = 1 << 8,
+  GDK_VULKAN_FEATURE_INCREMENTAL_PRESENT        = 1 << 9,
+  GDK_VULKAN_FEATURE_SWAPCHAIN_MAINTENANCE      = 1 << 10,
+  GDK_VULKAN_FEATURE_PORTABILITY_SUBSET         = 1 << 11,
 } GdkVulkanFeatures;
 
-#define GDK_VULKAN_N_FEATURES 6
+#define GDK_VULKAN_N_FEATURES 11
 
 #ifdef GDK_RENDERING_VULKAN
 extern const GdkDebugKey gdk_vulkan_feature_keys[];
@@ -137,13 +146,14 @@ struct _GdkDisplay
   guint have_egl_dma_buf_export : 1;
   guint have_egl_gl_colorspace : 1;
 
-  GdkDmabufFormats *dmabuf_formats;
+  /* atomic */ GdkDmabufFormats *dmabuf_formats;
   GdkDmabufDownloader *egl_downloader;
   GdkDmabufDownloader *vk_downloader;
 
    /* Cached data the EGL dmabuf downloader */
   GdkDmabufFormats *egl_dmabuf_formats;
   GdkDmabufFormats *egl_internal_formats;
+  GdkDmabufFormats *egl_external_formats;
 };
 
 struct _GdkDisplayClass
@@ -299,5 +309,7 @@ void gdk_display_set_cursor_theme          (GdkDisplay   *display,
                                             const char   *name,
                                             int           size);
 
+int gdk_display_guess_scale_factor         (GdkDisplay   *display);
+
 G_END_DECLS
 
