@@ -20,11 +20,13 @@
 
 #include "gdkdmabuftextureprivate.h"
 
+#include "gdkcolorstateprivate.h"
 #include "gdkdisplayprivate.h"
 #include "gdkdmabufdownloaderprivate.h"
 #include "gdkdmabufformatsbuilderprivate.h"
 #include "gdkdmabuffourccprivate.h"
 #include "gdkdmabufprivate.h"
+#include "gdkdmabuftexturebuilderprivate.h"
 #include "gdktextureprivate.h"
 #include <gdk/gdkglcontext.h>
 #include <gdk/gdkgltexturebuilder.h>
@@ -48,7 +50,6 @@ struct _GdkDmabufTexture
   GdkTexture parent_instance;
 
   GdkDisplay *display;
-  GdkDmabufDownloader *downloader;
 
   GdkDmabuf dmabuf;
 
@@ -61,6 +62,13 @@ struct _GdkDmabufTextureClass
   GdkTextureClass parent_class;
 };
 
+/**
+ * gdk_dmabuf_error_quark:
+ *
+ * Registers an error quark for [class@Gdk.DmabufTexture] errors.
+ *
+ * Returns: the error quark
+ **/
 G_DEFINE_QUARK (gdk-dmabuf-error-quark, gdk_dmabuf_error)
 
 G_DEFINE_TYPE (GdkDmabufTexture, gdk_dmabuf_texture, GDK_TYPE_TEXTURE)
@@ -76,7 +84,6 @@ gdk_dmabuf_texture_dispose (GObject *object)
       self->destroy = NULL;
     }
 
-  g_clear_object (&self->downloader);
   g_clear_object (&self->display);
 
   G_OBJECT_CLASS (gdk_dmabuf_texture_parent_class)->dispose (object);
@@ -87,9 +94,9 @@ typedef struct _Download Download;
 struct _Download
 {
   GdkDmabufTexture *texture;
-  GdkMemoryFormat format;
   guchar *data;
-  gsize stride;
+  GdkMemoryLayout layout;
+  GdkColorState *color_state;
   volatile int spinlock;
 };
 
@@ -97,34 +104,57 @@ static gboolean
 gdk_dmabuf_texture_invoke_callback (gpointer data)
 {
   Download *download = data;
+  GdkDisplay *display = download->texture->display;
 
-  gdk_dmabuf_downloader_download (download->texture->downloader,
-                                  download->texture,
-                                  download->format,
-                                  download->data,
-                                  download->stride);
+  if (display->egl_downloader &&
+      gdk_dmabuf_downloader_download (display->egl_downloader,
+                                      download->texture,
+                                      download->data,
+                                      &download->layout,
+                                      download->color_state))
+    {
+      /* Successfully downloaded using EGL */
+    }
+  else if (display->vk_downloader &&
+           gdk_dmabuf_downloader_download (display->vk_downloader,
+                                           download->texture,
+                                           download->data,
+                                           &download->layout,
+                                           download->color_state))
+    {
+      /* Successfully downloaded using Vulkan */
+    }
+#ifdef HAVE_DMABUF
+  else if (gdk_dmabuf_download_mmap (GDK_TEXTURE (download->texture),
+                                     download->data,
+                                     &download->layout,
+                                     download->color_state))
+    {
+      /* Successfully downloaded using mmap */
+    }
+#endif
+  else
+    {
+      const GdkDmabuf *dmabuf = gdk_dmabuf_texture_get_dmabuf (download->texture);
+
+      g_critical ("Failed to download %dx%d dmabuf texture (format %.4s:%#" G_GINT64_MODIFIER "x)",
+                  gdk_texture_get_width (GDK_TEXTURE (download->texture)),
+                  gdk_texture_get_height (GDK_TEXTURE (download->texture)),
+                  (char *)&(dmabuf->fourcc), dmabuf->modifier);
+    }
 
   g_atomic_int_set (&download->spinlock, 1);
-
   return FALSE;
 }
 
 static void
-gdk_dmabuf_texture_download (GdkTexture      *texture,
-                             GdkMemoryFormat  format,
-                             guchar          *data,
-                             gsize            stride)
+gdk_dmabuf_texture_download (GdkTexture            *texture,
+                             guchar                *data,
+                             const GdkMemoryLayout *layout,
+                             GdkColorState         *color_state)
 {
   GdkDmabufTexture *self = GDK_DMABUF_TEXTURE (texture);
-  Download download = { self, format, data, stride, 0 };
-
-  if (self->downloader == NULL)
-    {
-#ifdef HAVE_DMABUF
-      gdk_dmabuf_download_mmap (texture, format, data, stride);
-#endif
-      return;
-    }
+  Download download = { self, data, *layout, color_state, 0 };
 
   g_main_context_invoke (NULL, gdk_dmabuf_texture_invoke_callback, &download);
 
@@ -170,10 +200,11 @@ gdk_dmabuf_texture_new_from_builder (GdkDmabufTextureBuilder *builder,
   GdkTexture *update_texture;
   GdkDisplay *display;
   GdkDmabuf dmabuf;
-  GError *local_error = NULL;
+  GdkColorState *color_state;
   int width, height;
   gboolean premultiplied;
-  gsize i;
+  GdkMemoryFormat format;
+  gboolean is_yuv;
 
   display = gdk_dmabuf_texture_builder_get_display (builder);
   width = gdk_dmabuf_texture_builder_get_width (builder);
@@ -187,56 +218,65 @@ gdk_dmabuf_texture_new_from_builder (GdkDmabufTextureBuilder *builder,
                             error))
     return NULL;
 
+  if (gdk_memory_format_find_by_dmabuf_fourcc (dmabuf.fourcc, premultiplied, &format, &is_yuv))
+    {
+      if (!gdk_memory_format_is_block_boundary (format, width, height))
+        {
+          g_set_error (error,
+                       GDK_DMABUF_ERROR, GDK_DMABUF_ERROR_UNSUPPORTED_FORMAT,
+                       "Size of %dx%d is not valid for dmabuf format %.4s, must be multiple of %zux%zu",
+                       width, height,
+                       (char *) &dmabuf.fourcc,
+                       gdk_memory_format_get_block_width (format),
+                       gdk_memory_format_get_block_height (format));
+          return NULL;
+        }
+    }
+  else
+    {
+      g_set_error (error,
+                   GDK_DMABUF_ERROR, GDK_DMABUF_ERROR_UNSUPPORTED_FORMAT,
+                   "Unsupported dmabuf format %.4s",
+                   (char *) &dmabuf.fourcc);
+      return NULL;
+    }
+
   gdk_display_init_dmabuf (display);
 
+  if (!gdk_dmabuf_formats_contains (display->dmabuf_formats, dmabuf.fourcc, dmabuf.modifier))
+    {
+      g_set_error (error,
+                   GDK_DMABUF_ERROR, GDK_DMABUF_ERROR_UNSUPPORTED_FORMAT,
+                   "Unsupported dmabuf format: %.4s:%#" G_GINT64_MODIFIER "x",
+                   (char *) &dmabuf.fourcc, dmabuf.modifier);
+      return NULL;
+    }
+
+  color_state = gdk_dmabuf_texture_builder_get_color_state (builder);
+  if (color_state == NULL)
+    {
+      if (is_yuv)
+        color_state = GDK_COLOR_STATE_YUV;
+      else
+        color_state = GDK_COLOR_STATE_SRGB;
+    }
+
   self = g_object_new (GDK_TYPE_DMABUF_TEXTURE,
                        "width", width,
                        "height", height,
+                       "color-state", color_state,
                        NULL);
 
   g_set_object (&self->display, display);
+  GDK_TEXTURE (self)->format = format;
   self->dmabuf = dmabuf;
 
-  if (!gdk_dmabuf_get_memory_format (dmabuf.fourcc, premultiplied, &(GDK_TEXTURE (self)->format)))
-    {
-      GDK_DISPLAY_DEBUG (display, DMABUF,
-                         "Falling back to generic RGBA for dmabuf format %.4s",
-                         (char *) &dmabuf.fourcc);
-      GDK_TEXTURE (self)->format = premultiplied ? GDK_MEMORY_R8G8B8A8_PREMULTIPLIED
-                                                 : GDK_MEMORY_R8G8B8A8;
-    }
-
-  if (!gdk_dmabuf_formats_contains (gdk_dmabuf_get_mmap_formats (), dmabuf.fourcc, dmabuf.modifier))
-    {
-      for (i = 0; display->dmabuf_downloaders[i] != NULL; i++)
-        {
-          if (local_error && g_error_matches (local_error, GDK_DMABUF_ERROR, GDK_DMABUF_ERROR_UNSUPPORTED_FORMAT))
-            g_clear_error (&local_error);
-
-          if (gdk_dmabuf_downloader_supports (display->dmabuf_downloaders[i],
-                                              self,
-                                              local_error ? NULL : &local_error))
-            {
-              self->downloader = g_object_ref (display->dmabuf_downloaders[i]);
-              break;
-            }
-        }
-
-      if (self->downloader == NULL)
-        {
-          g_propagate_error (error, local_error);
-          g_object_unref (self);
-          return NULL;
-        }
-    }
-
   GDK_DISPLAY_DEBUG (display, DMABUF,
-                     "Creating dmabuf texture, format %.4s:%#" G_GINT64_MODIFIER "x, %s%u planes, memory format %u, downloader %s",
+                     "Creating dmabuf texture, format %.4s:%#" G_GINT64_MODIFIER "x, %s%u planes, memory format %s",
                      (char *) &dmabuf.fourcc, dmabuf.modifier,
                      gdk_dmabuf_texture_builder_get_premultiplied (builder) ? " premultiplied, " : "",
                      dmabuf.n_planes,
-                     GDK_TEXTURE (self)->format,
-                     self->downloader ? G_OBJECT_TYPE_NAME (self->downloader) : "none");
+                     gdk_memory_format_get_name (GDK_TEXTURE (self)->format));
 
   /* Set this only once we know that the texture will be created.
    * Otherwise dispose() will run the callback */
