@@ -25,7 +25,7 @@
 /**
  * GdkDrag:
  *
- * The `GdkDrag` object represents the source of an ongoing DND operation.
+ * Represents the source of an ongoing DND operation.
  *
  * A `GdkDrag` is created when a drag is started, and stays alive for duration of
  * the DND operation. After a drag has been started with [func@Gdk.Drag.begin],
@@ -92,7 +92,7 @@ static GList *drags = NULL;
 G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GdkDrag, gdk_drag, G_TYPE_OBJECT)
 
 /**
- * gdk_drag_get_display: (attributes org.gtk.Method.get_property=display)
+ * gdk_drag_get_display:
  * @drag: a `GdkDrag`
  *
  * Gets the `GdkDisplay` that the drag object was created for.
@@ -110,7 +110,7 @@ gdk_drag_get_display (GdkDrag *drag)
 }
 
 /**
- * gdk_drag_get_formats: (attributes org.gtk.Method.get_property=formats)
+ * gdk_drag_get_formats:
  * @drag: a `GdkDrag`
  *
  * Retrieves the formats supported by this `GdkDrag` object.
@@ -128,7 +128,7 @@ gdk_drag_get_formats (GdkDrag *drag)
 }
 
 /**
- * gdk_drag_get_actions: (attributes org.gtk.Method.get_property=actions)
+ * gdk_drag_get_actions:
  * @drag: a `GdkDrag`
  *
  * Determines the bitmask of possible actions proposed by the source.
@@ -140,13 +140,13 @@ gdk_drag_get_actions (GdkDrag *drag)
 {
   GdkDragPrivate *priv = gdk_drag_get_instance_private (drag);
 
-  g_return_val_if_fail (GDK_IS_DRAG (drag), 0);
+  g_return_val_if_fail (GDK_IS_DRAG (drag), GDK_ACTION_NONE);
 
   return priv->actions;
 }
 
 /**
- * gdk_drag_get_selected_action: (attributes org.gtk.Method.get_property=selected-action)
+ * gdk_drag_get_selected_action:
  * @drag: a `GdkDrag`
  *
  * Determines the action chosen by the drag destination.
@@ -158,13 +158,13 @@ gdk_drag_get_selected_action (GdkDrag *drag)
 {
   GdkDragPrivate *priv = gdk_drag_get_instance_private (drag);
 
-  g_return_val_if_fail (GDK_IS_DRAG (drag), 0);
+  g_return_val_if_fail (GDK_IS_DRAG (drag), GDK_ACTION_NONE);
 
   return priv->selected_action;
 }
 
 /**
- * gdk_drag_get_device: (attributes org.gtk.Method.get_property=device)
+ * gdk_drag_get_device:
  * @drag: a `GdkDrag`
  *
  * Returns the `GdkDevice` associated to the `GdkDrag` object.
@@ -182,7 +182,7 @@ gdk_drag_get_device (GdkDrag *drag)
 }
 
 /**
- * gdk_drag_get_content: (attributes org.gtk.Method.get_property=content)
+ * gdk_drag_get_content:
  * @drag: a `GdkDrag`
  *
  * Returns the `GdkContentProvider` associated to the `GdkDrag` object.
@@ -200,7 +200,7 @@ gdk_drag_get_content (GdkDrag *drag)
 }
 
 /**
- * gdk_drag_get_surface: (attributes org.gtk.Method.get_property=surface)
+ * gdk_drag_get_surface:
  * @drag: a `GdkDrag`
  *
  * Returns the `GdkSurface` where the drag originates.
@@ -362,7 +362,7 @@ gdk_drag_class_init (GdkDragClass *klass)
   object_class->finalize = gdk_drag_finalize;
 
   /**
-   * GdkDrag:content: (attributes org.gtk.Property.get=gdk_drag_get_content)
+   * GdkDrag:content:
    *
    * The `GdkContentProvider`.
    */
@@ -375,7 +375,7 @@ gdk_drag_class_init (GdkDragClass *klass)
                          G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GdkDrag:device: (attributes org.gtk.Property.get=gdk_drag_get_device)
+   * GdkDrag:device:
    *
    * The `GdkDevice` that is performing the drag.
    */
@@ -388,7 +388,7 @@ gdk_drag_class_init (GdkDragClass *klass)
                          G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GdkDrag:display: (attributes org.gtk.Property.get=gdk_drag_get_display)
+   * GdkDrag:display:
    *
    * The `GdkDisplay` that the drag belongs to.
    */
@@ -400,7 +400,7 @@ gdk_drag_class_init (GdkDragClass *klass)
                          G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GdkDrag:formats: (attributes org.gtk.Property.get=gdk_drag_get_formats)
+   * GdkDrag:formats:
    *
    * The possible formats that the drag can provide its data in.
    */
@@ -413,7 +413,7 @@ gdk_drag_class_init (GdkDragClass *klass)
                         G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GdkDrag:selected-action: (attributes org.gtk.Property.get=gdk_drag_get_selected_action)
+   * GdkDrag:selected-action:
    *
    * The currently selected action of the drag.
    */
@@ -426,20 +426,20 @@ gdk_drag_class_init (GdkDragClass *klass)
                         G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GdkDrag:actions: (attributes org.gtk.Property.get=gdk_drag_get_actions)
+   * GdkDrag:actions:
    *
    * The possible actions of this drag.
    */
   properties[PROP_ACTIONS] =
     g_param_spec_flags ("actions", NULL, NULL,
                         GDK_TYPE_DRAG_ACTION,
-                        0,
+                        GDK_ACTION_NONE,
                         G_PARAM_READWRITE |
                         G_PARAM_STATIC_STRINGS |
                         G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GdkDrag:surface: (attributes org.gtk.Property.get=gdk_drag_get_surface)
+   * GdkDrag:surface:
    *
    * The surface where the drag originates.
    */
@@ -641,15 +641,13 @@ gdk_drag_set_selected_action (GdkDrag       *drag,
                               GdkDragAction  action)
 {
   GdkDragPrivate *priv = gdk_drag_get_instance_private (drag);
-  GdkCursor *cursor;
 
   if (priv->selected_action == action)
     return;
 
   priv->selected_action = action;
 
-  cursor = gdk_drag_get_cursor (drag, action);
-  gdk_drag_set_cursor (drag, cursor);
+  gdk_drag_update_cursor (drag);
 
   g_object_notify_by_pspec (G_OBJECT (drag), properties[PROP_SELECTED_ACTION]);
 }
@@ -736,13 +734,12 @@ gdk_drag_drop_done (GdkDrag  *drag,
 }
 
 void
-gdk_drag_set_cursor (GdkDrag   *drag,
-                     GdkCursor *cursor)
+gdk_drag_update_cursor (GdkDrag *drag)
 {
   g_return_if_fail (GDK_IS_DRAG (drag));
 
-  if (GDK_DRAG_GET_CLASS (drag)->set_cursor)
-    GDK_DRAG_GET_CLASS (drag)->set_cursor (drag, cursor);
+  if (GDK_DRAG_GET_CLASS (drag)->update_cursor)
+    GDK_DRAG_GET_CLASS (drag)->update_cursor (drag);
 }
 
 void
@@ -811,7 +808,7 @@ gdk_drag_get_cursor (GdkDrag       *drag,
  * Checks if @action represents a single action or includes
  * multiple actions.
  *
- * When @action is 0 - ie no action was given, %TRUE
+ * When @action is `GDK_ACTION_NONE` - ie no action was given, `TRUE`
  * is returned.
  *
  * Returns: %TRUE if exactly one action was given
