@@ -34,6 +34,28 @@
 
 G_BEGIN_DECLS
 
+/**
+ * GdkColorChannel:
+ * @GDK_COLOR_CHANNEL_RED: The red color channel, aka 0
+ * @GDK_COLOR_CHANNEL_GREEN: The green color channel, aka 1
+ * @GDK_COLOR_CHANNEL_BLUE: The blue color channel, aka 2
+ * @GDK_COLOR_CHANNEL_ALPHA: The alpha color channel, aka 3
+ *
+ * Enumerates the color channels of RGBA values as used in
+ * `GdkColor` and OpenGL/Vulkan shaders.
+ *
+ * Note that this is not the order of pixel values in Cairo
+ * and `GdkMemoryFormat` can have many different orders.
+ *
+ * Since: 4.22
+ */
+typedef enum {
+  GDK_COLOR_CHANNEL_RED GDK_AVAILABLE_ENUMERATOR_IN_4_22 = 0,
+  GDK_COLOR_CHANNEL_GREEN GDK_AVAILABLE_ENUMERATOR_IN_4_22 = 1,
+  GDK_COLOR_CHANNEL_BLUE GDK_AVAILABLE_ENUMERATOR_IN_4_22 = 2,
+  GDK_COLOR_CHANNEL_ALPHA GDK_AVAILABLE_ENUMERATOR_IN_4_22 = 3
+} GdkColorChannel;
+
 /**
  * GdkGLAPI:
  * @GDK_GL_API_GL: The OpenGL API
