@@ -63,6 +63,9 @@
 #include "presentation-time-client-protocol.h"
 #include "color-management-v1-client-protocol.h"
 #include "color-representation-v1-client-protocol.h"
+#include "xx-session-management-v1-client-protocol.h"
+
+#include "wm-button-layout-translation.h"
 
 #include "gdk/gdkprivate.h"
 
@@ -177,6 +180,9 @@ init_skip_protocols (GdkWaylandDisplay *display_wayland)
   if (!GDK_DISPLAY_DEBUG_CHECK (GDK_DISPLAY (display_wayland), COLOR_MANAGEMENT))
     g_strv_builder_add (strv, wp_color_manager_v1_interface.name);
 
+  if (!GDK_DISPLAY_DEBUG_CHECK (GDK_DISPLAY (display_wayland), SESSION_MANAGEMENT))
+    g_strv_builder_add (strv, xx_session_manager_v1_interface.name);
+
   if (help)
     {
       gdk_help_message ("%s can be set to a list of Wayland interfaces to disable.\n", variable);
@@ -455,6 +461,49 @@ static const struct org_kde_kwin_server_decoration_manager_listener server_decor
   .default_mode = server_decoration_manager_default_mode
 };
 
+/* }}} */
+/* {{{ session listener */
+
+static void
+session_listener_created (void                 *data,
+                          struct xx_session_v1 *xx_session_v1,
+                          const char           *id)
+{
+  GdkWaylandDisplay *display_wayland = data;
+
+  GDK_DEBUG (MISC, "session created: %s", id);
+
+  if (g_strcmp0 (display_wayland->session_id, id) != 0)
+    {
+      g_clear_pointer (&display_wayland->session_id, g_free);
+      display_wayland->session_id = g_strdup (id);
+    }
+}
+
+static void
+session_listener_restored (void                 *data,
+                           struct xx_session_v1 *xx_session_v1)
+{
+  GdkWaylandDisplay *display_wayland = data;
+
+  GDK_DEBUG (MISC, "session restored: %s", display_wayland->session_id);
+}
+
+static void
+session_listener_replaced (void                 *data,
+                           struct xx_session_v1 *xx_session_v1)
+{
+  GdkWaylandDisplay *display_wayland = data;
+
+  GDK_DEBUG (MISC, "session replaced: %s", display_wayland->session_id);
+}
+
+static const struct xx_session_v1_listener xx_session_listener = {
+  .created = session_listener_created,
+  .restored = session_listener_restored,
+  .replaced = session_listener_replaced,
+};
+
 /* }}} */
 /* {{{ wl_registry listener */
 
@@ -515,6 +564,13 @@ gdk_registry_handle_global (void               *data,
                           &xdg_wm_dialog_v1_interface,
                           MIN (version, XDG_WM_DIALOG_VERSION));
     }
+  else if (match_global (display_wayland, interface, version, xx_session_manager_v1_interface.name, 0))
+    {
+      display_wayland->xx_session_manager =
+        wl_registry_bind (display_wayland->wl_registry, id,
+                          &xx_session_manager_v1_interface,
+                          version);
+    }
   else if (match_global (display_wayland, interface, version, gtk_shell1_interface.name, 0))
     {
       display_wayland->gtk_shell =
@@ -833,11 +889,12 @@ gdk_wayland_display_dispose (GObject *object)
 {
   GdkWaylandDisplay *display_wayland = GDK_WAYLAND_DISPLAY (object);
 
-  g_list_free_full (display_wayland->toplevels, destroy_toplevel);
+  g_clear_list (&display_wayland->toplevels, destroy_toplevel);
 
   gdk_wayland_display_uninstall_gsources (display_wayland);
 
-  g_list_free_full (display_wayland->async_roundtrips, (GDestroyNotify) wl_callback_destroy);
+  g_clear_list (&display_wayland->async_roundtrips,
+                (GDestroyNotify) wl_callback_destroy);
 
   if (display_wayland->known_globals)
     {
@@ -845,7 +902,7 @@ gdk_wayland_display_dispose (GObject *object)
       display_wayland->known_globals = NULL;
     }
 
-  g_list_free_full (display_wayland->on_has_globals_closures, g_free);
+  g_clear_list (&display_wayland->on_has_globals_closures, g_free);
 
   g_clear_pointer (&display_wayland->compositor, wl_compositor_destroy);
   g_clear_pointer (&display_wayland->xdg_wm_base, xdg_wm_base_destroy);
@@ -874,6 +931,8 @@ gdk_wayland_display_dispose (GObject *object)
   g_clear_pointer (&display_wayland->color, gdk_wayland_color_free);
   g_clear_pointer (&display_wayland->system_bell, xdg_system_bell_v1_destroy);
   g_clear_pointer (&display_wayland->toplevel_icon, xdg_toplevel_icon_manager_v1_destroy);
+  g_clear_pointer (&display_wayland->xx_session, xx_session_v1_destroy);
+  g_clear_pointer (&display_wayland->xx_session_manager, xx_session_manager_v1_destroy);
 
   g_clear_pointer (&display_wayland->shm, wl_shm_destroy);
   g_clear_pointer (&display_wayland->wl_registry, wl_registry_destroy);
@@ -895,9 +954,6 @@ gdk_wayland_display_finalize (GObject *object)
   g_free (display_wayland->startup_notification_id);
   xkb_context_unref (display_wayland->xkb_context);
 
-  if (display_wayland->settings)
-    g_hash_table_destroy (display_wayland->settings);
-
   g_clear_object (&display_wayland->settings_portal);
 
   g_strfreev (display_wayland->skip_protocols);
@@ -1321,6 +1377,55 @@ gdk_wayland_display_query_registry (GdkDisplay *display,
   return FALSE;
 }
 
-/* }}} */
+gboolean
+gdk_wayland_display_register_session (GdkDisplay                        *display,
+                                      enum xx_session_manager_v1_reason  reason,
+                                      const char                        *name)
+{
+  GdkWaylandDisplay *display_wayland = GDK_WAYLAND_DISPLAY (display);
 
+  GDK_DEBUG (MISC, "register session %s, reason %u", name, reason);
+
+  if (!display_wayland->xx_session_manager)
+    return FALSE;
+
+  g_clear_pointer (&display_wayland->session_id, g_free);
+  display_wayland->session_id = g_strdup (name);
+
+  display_wayland->xx_session =
+    xx_session_manager_v1_get_session (display_wayland->xx_session_manager,
+                                       reason,
+                                       name);
+  xx_session_v1_add_listener (display_wayland->xx_session,
+                              &xx_session_listener,
+                              display_wayland);
+
+  wl_display_roundtrip (display_wayland->wl_display);
+  return TRUE;
+}
+
+void
+gdk_wayland_display_unregister_session (GdkDisplay *display)
+{
+  GdkWaylandDisplay *display_wayland = GDK_WAYLAND_DISPLAY (display);
+
+  if (!display_wayland->xx_session_manager)
+    return;
+
+  if (display_wayland->xx_session)
+    xx_session_v1_remove (display_wayland->xx_session);
+
+  g_clear_pointer (&display_wayland->session_id, g_free);
+  g_clear_pointer (&display_wayland->xx_session, xx_session_v1_destroy);
+}
+
+const char *
+gdk_wayland_display_get_session_id (GdkDisplay *display)
+{
+  GdkWaylandDisplay *display_wayland = GDK_WAYLAND_DISPLAY (display);
+
+  return display_wayland->session_id;
+}
+
+/* }}} */
 /* vim:set foldmethod=marker: */
