@@ -25,7 +25,7 @@
 #include "gdkdevice-winpointer.h"
 #include "gdkdeviceprivate.h"
 #include "gdkdevicetoolprivate.h"
-#include "gdkdisplayprivate.h"
+#include "gdkdisplay-win32.h"
 #include "gdkeventsprivate.h"
 #include "gdkseatdefaultprivate.h"
 
@@ -65,40 +65,44 @@ typedef BOOL
 typedef BOOL
 (WINAPI *setWindowFeedbackSetting_t)(HWND hwnd, FEEDBACK_TYPE feedback, DWORD dwFlags, UINT32 size, const VOID *configuration);
 
-static registerPointerDeviceNotifications_t registerPointerDeviceNotifications;
-static getPointerDevices_t getPointerDevices;
-static getPointerDeviceCursors_t getPointerDeviceCursors;
-static getPointerDeviceRects_t getPointerDeviceRects;
-static getPointerType_t getPointerType;
-static getPointerCursorId_t getPointerCursorId;
-static getPointerPenInfo_t getPointerPenInfo;
-static getPointerTouchInfo_t getPointerTouchInfo;
-static getPointerPenInfoHistory_t getPointerPenInfoHistory;
-static getPointerTouchInfoHistory_t getPointerTouchInfoHistory;
-static setGestureConfig_t setGestureConfig;
-static setWindowFeedbackSetting_t setWindowFeedbackSetting;
+struct _GdkDeviceManagerWin32WinpointerFuncs
+{
+  registerPointerDeviceNotifications_t registerPointerDeviceNotifications;
+  getPointerDevices_t getPointerDevices;
+  getPointerDeviceCursors_t getPointerDeviceCursors;
+  getPointerDeviceRects_t getPointerDeviceRects;
+  getPointerType_t getPointerType;
+  getPointerCursorId_t getPointerCursorId;
+  getPointerPenInfo_t getPointerPenInfo;
+  getPointerTouchInfo_t getPointerTouchInfo;
+  getPointerPenInfoHistory_t getPointerPenInfoHistory;
+  getPointerTouchInfoHistory_t getPointerTouchInfoHistory;
+  setGestureConfig_t setGestureConfig;
+  setWindowFeedbackSetting_t setWindowFeedbackSetting;
+};
+typedef struct _GdkDeviceManagerWin32WinpointerFuncs GdkDeviceManagerWin32WinpointerFuncs;
 
-static ATOM notifications_window_class;
-static HWND notifications_window_handle;
-
-static GPtrArray *ignored_interactions;
+#define WINPOINTER_API(dm,f) ((GdkDeviceManagerWin32WinpointerFuncs *)dm->winpointer_funcs)->f
 
 static inline void
-winpointer_ignore_interaction (UINT32 pointer_id)
+winpointer_ignore_interaction (GdkDeviceManagerWin32 *device_manager,
+                               UINT32                 pointer_id)
 {
-  g_ptr_array_add (ignored_interactions, GUINT_TO_POINTER (pointer_id));
+  g_ptr_array_add (device_manager->ignored_interactions, GUINT_TO_POINTER (pointer_id));
 }
 
 static inline void
-winpointer_remove_ignored_interaction (UINT32 pointer_id)
+winpointer_remove_ignored_interaction (GdkDeviceManagerWin32 *device_manager,
+                                       UINT32                 pointer_id)
 {
-  g_ptr_array_remove_fast (ignored_interactions, GUINT_TO_POINTER (pointer_id));
+  g_ptr_array_remove_fast (device_manager->ignored_interactions, GUINT_TO_POINTER (pointer_id));
 }
 
 static inline gboolean
-winpointer_should_ignore_interaction (UINT32 pointer_id)
+winpointer_should_ignore_interaction (GdkDeviceManagerWin32 *device_manager,
+                                      UINT32                 pointer_id)
 {
-  return g_ptr_array_find (ignored_interactions, GUINT_TO_POINTER (pointer_id), NULL);
+  return g_ptr_array_find (device_manager->ignored_interactions, GUINT_TO_POINTER (pointer_id), NULL);
 }
 
 static inline guint32
@@ -136,11 +140,12 @@ copy_axes (double *axes)
 }
 
 static GdkDeviceWinpointer*
-winpointer_find_device_with_source (HANDLE device_handle,
-                                    UINT32 cursor_id,
-                                    GdkInputSource input_source)
+winpointer_find_device_with_source (GdkDeviceManagerWin32 *device_manager,
+                                    HANDLE                 device_handle,
+                                    UINT32                 cursor_id,
+                                    GdkInputSource         input_source)
 {
-  for (GList *l = _gdk_device_manager->winpointer_devices; l != NULL; l = l->next)
+  for (GList *l = device_manager->winpointer_devices; l != NULL; l = l->next)
     {
       GdkDeviceWinpointer *device = (GdkDeviceWinpointer*) l->data;
 
@@ -157,9 +162,10 @@ winpointer_find_device_with_source (HANDLE device_handle,
 }
 
 static gboolean
-winpointer_get_event_type (MSG *msg,
-                           POINTER_INFO *info,
-                           GdkEventType *evt_type)
+winpointer_get_event_type (GdkDeviceManagerWin32 *device_manager,
+                           MSG                   *msg,
+                           POINTER_INFO          *info,
+                           GdkEventType          *evt_type)
 {
   switch (info->pointerType)
     {
@@ -189,7 +195,7 @@ winpointer_get_event_type (MSG *msg,
       if (IS_POINTER_CANCELED_WPARAM (msg->wParam) ||
           !HAS_POINTER_CONFIDENCE_WPARAM (msg->wParam))
         {
-          winpointer_ignore_interaction (GET_POINTERID_WPARAM (msg->wParam));
+          winpointer_ignore_interaction (device_manager, GET_POINTERID_WPARAM (msg->wParam));
 
           if (((info->pointerFlags & POINTER_FLAG_INCONTACT) &&
                (info->pointerFlags & POINTER_FLAG_UPDATE)) ||
@@ -249,10 +255,11 @@ winpointer_make_event (GdkDeviceWinpointer *device,
   GdkEventType evt_type;
   GdkEvent *evt = NULL;
   GdkDevice *core_device = NULL;
+  GdkDeviceManagerWin32 *device_manager = GDK_WIN32_DISPLAY (gdk_surface_get_display (surface))->device_manager;
 
-  core_device = _gdk_device_manager->core_pointer;
+  core_device = device_manager->core_pointer;
 
-  if (!winpointer_get_event_type (msg, info, &evt_type))
+  if (!winpointer_get_event_type (device_manager, msg, info, &evt_type))
     return;
 
   time = winpointer_get_time (msg, info);
@@ -399,14 +406,16 @@ gdk_winpointer_input_events (GdkSurface *surface,
   UINT32 pointer_id = GET_POINTERID_WPARAM (msg->wParam);
   POINTER_INPUT_TYPE type = PT_POINTER;
   UINT32 cursor_id = 0;
+  GdkWin32Display *display_win32 = GDK_WIN32_DISPLAY (gdk_surface_get_display (surface));
+  GdkDeviceManagerWin32 *device_manager = display_win32->device_manager;
 
-  if (!getPointerType (pointer_id, &type))
+  if (!WINPOINTER_API (device_manager, getPointerType) (pointer_id, &type))
     {
       WIN32_API_FAILED_LOG_ONCE ("GetPointerType");
       return;
     }
 
-  if (!getPointerCursorId (pointer_id, &cursor_id))
+  if (!WINPOINTER_API (device_manager, getPointerCursorId) (pointer_id, &cursor_id))
     {
       WIN32_API_FAILED_LOG_ONCE ("GetPointerCursorId");
       return;
@@ -415,7 +424,7 @@ gdk_winpointer_input_events (GdkSurface *surface,
   if (winpointer_should_filter_message (msg, type))
     return;
 
-  if (winpointer_should_ignore_interaction (pointer_id))
+  if (winpointer_should_ignore_interaction (device_manager, pointer_id))
     return;
 
   switch (type)
@@ -431,7 +440,7 @@ gdk_winpointer_input_events (GdkSurface *surface,
         do
           {
             infos = g_new0 (POINTER_PEN_INFO, history_count);
-            if (!getPointerPenInfoHistory (pointer_id, &history_count, infos))
+            if (!WINPOINTER_API (device_manager, getPointerPenInfoHistory) (pointer_id, &history_count, infos))
               {
                 WIN32_API_FAILED_LOG_ONCE ("GetPointerPenInfoHistory");
                 g_free (infos);
@@ -443,7 +452,7 @@ gdk_winpointer_input_events (GdkSurface *surface,
         if (G_UNLIKELY (history_count == 0))
           return;
 
-        device = winpointer_find_device_with_source (infos->pointerInfo.sourceDevice, cursor_id, GDK_SOURCE_PEN);
+        device = winpointer_find_device_with_source (device_manager, infos->pointerInfo.sourceDevice, cursor_id, GDK_SOURCE_PEN);
         if (G_UNLIKELY (!device))
           {
             g_free (infos);
@@ -484,7 +493,7 @@ gdk_winpointer_input_events (GdkSurface *surface,
         do
           {
             infos = g_new0 (POINTER_TOUCH_INFO, history_count);
-            if (!getPointerTouchInfoHistory (pointer_id, &history_count, infos))
+            if (!WINPOINTER_API (device_manager, getPointerTouchInfoHistory) (pointer_id, &history_count, infos))
               {
                 WIN32_API_FAILED_LOG_ONCE ("GetPointerTouchInfoHistory");
                 g_free (infos);
@@ -496,7 +505,7 @@ gdk_winpointer_input_events (GdkSurface *surface,
         if (G_UNLIKELY (history_count == 0))
           return;
 
-        device = winpointer_find_device_with_source (infos->pointerInfo.sourceDevice, cursor_id, GDK_SOURCE_TOUCHSCREEN);
+        device = winpointer_find_device_with_source (device_manager, infos->pointerInfo.sourceDevice, cursor_id, GDK_SOURCE_TOUCHSCREEN);
         if (G_UNLIKELY (!device))
           {
             g_free (infos);
@@ -524,21 +533,23 @@ gdk_winpointer_input_events (GdkSurface *surface,
 }
 
 gboolean
-gdk_winpointer_get_message_info (MSG *msg,
-                                 GdkDevice **device,
-                                 guint32 *time)
+gdk_winpointer_get_message_info (MSG              *msg,
+                                 GdkDevice       **device,
+                                 GdkWin32Display  *display_win32,
+                                 guint32          *time)
 {
   UINT32 pointer_id = GET_POINTERID_WPARAM (msg->wParam);
   POINTER_INPUT_TYPE type = PT_POINTER;
   UINT32 cursor_id = 0;
+  GdkDeviceManagerWin32 *device_manager = display_win32->device_manager;
 
-  if (!getPointerType (pointer_id, &type))
+  if (!WINPOINTER_API (device_manager, getPointerType) (pointer_id, &type))
     {
       WIN32_API_FAILED_LOG_ONCE ("GetPointerType");
       return FALSE;
     }
 
-  if (!getPointerCursorId (pointer_id, &cursor_id))
+  if (!WINPOINTER_API (device_manager, getPointerCursorId) (pointer_id, &cursor_id))
     {
       WIN32_API_FAILED_LOG_ONCE ("GetPointerCursorId");
       return FALSE;
@@ -550,13 +561,13 @@ gdk_winpointer_get_message_info (MSG *msg,
       {
         POINTER_PEN_INFO pen_info;
 
-        if (!getPointerPenInfo (pointer_id, &pen_info))
+        if (!WINPOINTER_API (device_manager, getPointerPenInfo) (pointer_id, &pen_info))
           {
             WIN32_API_FAILED_LOG_ONCE ("GetPointerPenInfo");
             return FALSE;
           }
 
-        *device = (GdkDevice*) winpointer_find_device_with_source (pen_info.pointerInfo.sourceDevice, cursor_id, GDK_SOURCE_PEN);
+        *device = (GdkDevice*) winpointer_find_device_with_source (device_manager, pen_info.pointerInfo.sourceDevice, cursor_id, GDK_SOURCE_PEN);
         *time = winpointer_get_time (msg, &pen_info.pointerInfo);
       }
     break;
@@ -564,13 +575,14 @@ gdk_winpointer_get_message_info (MSG *msg,
       {
         POINTER_TOUCH_INFO touch_info;
 
-        if (!getPointerTouchInfo (pointer_id, &touch_info))
+        if (!WINPOINTER_API (device_manager, getPointerTouchInfo) (pointer_id, &touch_info))
             {
               WIN32_API_FAILED_LOG_ONCE ("GetPointerTouchInfo");
               return FALSE;
             }
 
-        *device = GDK_DEVICE (winpointer_find_device_with_source (touch_info.pointerInfo.sourceDevice,
+        *device = GDK_DEVICE (winpointer_find_device_with_source (device_manager,
+                                                                  touch_info.pointerInfo.sourceDevice,
                                                                   cursor_id,
                                                                   GDK_SOURCE_TOUCHSCREEN));
 
@@ -587,12 +599,13 @@ gdk_winpointer_get_message_info (MSG *msg,
 }
 
 gboolean
-gdk_winpointer_should_forward_message (MSG *msg)
+gdk_winpointer_should_forward_message (GdkDeviceManagerWin32 *device_manager,
+                                       MSG                   *msg)
 {
   UINT32 pointer_id = GET_POINTERID_WPARAM (msg->wParam);
   POINTER_INPUT_TYPE type = PT_POINTER;
 
-  if (!getPointerType (pointer_id, &type))
+  if (!WINPOINTER_API (device_manager, getPointerType) (pointer_id, &type))
     {
       WIN32_API_FAILED_LOG_ONCE ("GetPointerType");
       return TRUE;
@@ -602,9 +615,10 @@ gdk_winpointer_should_forward_message (MSG *msg)
 }
 
 void
-gdk_winpointer_interaction_ended (MSG *msg)
+gdk_winpointer_interaction_ended (GdkDeviceManagerWin32 *device_manager,
+                                  MSG                   *msg)
 {
-  winpointer_remove_ignored_interaction (GET_POINTERID_WPARAM (msg->wParam));
+  winpointer_remove_ignored_interaction (device_manager, GET_POINTERID_WPARAM (msg->wParam));
 }
 
 static inline double
@@ -626,12 +640,13 @@ utils_rect_is_degenerate (RECT *rect)
 }
 
 static gboolean
-winpointer_device_update_scale_factors (GdkDeviceWinpointer *device)
+winpointer_device_update_scale_factors (GdkDeviceWinpointer   *device,
+                                        GdkDeviceManagerWin32 *device_manager)
 {
   RECT device_rect;
   RECT display_rect;
 
-  if (!getPointerDeviceRects (device->device_handle, &device_rect, &display_rect))
+  if (!WINPOINTER_API (device_manager, getPointerDeviceRects) (device->device_handle, &device_rect, &display_rect))
     {
       WIN32_API_FAILED ("GetPointerDeviceRects");
       return FALSE;
@@ -716,8 +731,9 @@ winpointer_get_device_details (HANDLE device,
 }
 
 static void
-winpointer_create_device (POINTER_DEVICE_INFO *info,
-                          GdkInputSource source)
+winpointer_create_device (GdkDeviceManagerWin32 *device_manager,
+                          POINTER_DEVICE_INFO   *info,
+                          GdkInputSource         source)
 {
   GdkDeviceWinpointer *device = NULL;
   GdkSeat *seat = NULL;
@@ -731,12 +747,12 @@ winpointer_create_device (POINTER_DEVICE_INFO *info,
   UINT32 num_cursors = 0;
   GdkAxisFlags axes_flags = 0;
 
-  seat = gdk_display_get_default_seat (_gdk_display);
+  seat = gdk_display_get_default_seat (device_manager->display);
 
   memset (pid, 0, VID_PID_CHARS + 1);
   memset (vid, 0, VID_PID_CHARS + 1);
 
-  if (!getPointerDeviceCursors (info->device, &num_cursors, NULL))
+  if (!WINPOINTER_API (device_manager, getPointerDeviceCursors) (info->device, &num_cursors, NULL))
     {
       WIN32_API_FAILED ("GetPointerDeviceCursors");
       return;
@@ -780,7 +796,7 @@ winpointer_create_device (POINTER_DEVICE_INFO *info,
     }
 
   device = g_object_new (GDK_TYPE_DEVICE_WINPOINTER,
-                         "display", _gdk_display,
+                         "display", device_manager->display,
                          "seat", seat,
                          "has-cursor", TRUE,
                          "source", source,
@@ -820,7 +836,7 @@ winpointer_create_device (POINTER_DEVICE_INFO *info,
   device->start_cursor_id = info->startingCursorId;
   device->end_cursor_id = info->startingCursorId + num_cursors - 1;
 
-  if (!winpointer_device_update_scale_factors (device))
+  if (!winpointer_device_update_scale_factors (device, device_manager))
     {
       g_set_object (&device, NULL);
       goto cleanup;
@@ -844,10 +860,10 @@ winpointer_create_device (POINTER_DEVICE_INFO *info,
     break;
     }
 
-  _gdk_device_manager->winpointer_devices = g_list_append (_gdk_device_manager->winpointer_devices, device);
+  device_manager->winpointer_devices = g_list_append (device_manager->winpointer_devices, device);
 
-  _gdk_device_set_associated_device (GDK_DEVICE (device), _gdk_device_manager->core_pointer);
-  _gdk_device_add_physical_device (_gdk_device_manager->core_pointer, GDK_DEVICE (device));
+  _gdk_device_set_associated_device (GDK_DEVICE (device), device_manager->core_pointer);
+  _gdk_device_add_physical_device (device_manager->core_pointer, GDK_DEVICE (device));
 
   gdk_seat_default_add_physical_device (GDK_SEAT_DEFAULT (seat), GDK_DEVICE (device));
 
@@ -859,16 +875,17 @@ cleanup:
 }
 
 static void
-winpointer_create_devices (POINTER_DEVICE_INFO *info)
+winpointer_create_devices (GdkDeviceManagerWin32 *device_manager,
+                           POINTER_DEVICE_INFO   *info)
 {
   switch (info->pointerDeviceType)
     {
     case POINTER_DEVICE_TYPE_INTEGRATED_PEN:
     case POINTER_DEVICE_TYPE_EXTERNAL_PEN:
-      winpointer_create_device (info, GDK_SOURCE_PEN);
+      winpointer_create_device (device_manager, info, GDK_SOURCE_PEN);
     break;
     case POINTER_DEVICE_TYPE_TOUCH:
-      winpointer_create_device (info, GDK_SOURCE_TOUCHSCREEN);
+      winpointer_create_device (device_manager, info, GDK_SOURCE_TOUCHSCREEN);
     break;
     default:
       g_warn_if_reached ();
@@ -894,9 +911,10 @@ winpointer_find_device_in_system_list (GdkDeviceWinpointer *device,
 }
 
 static gboolean
-winpointer_find_system_device_in_device_manager (POINTER_DEVICE_INFO *info)
+winpointer_find_system_device_in_device_manager (GdkDeviceManagerWin32 *device_manager,
+                                                 POINTER_DEVICE_INFO   *info)
 {
-  for (GList *l = _gdk_device_manager->winpointer_devices; l != NULL; l = l->next)
+  for (GList *l = device_manager->winpointer_devices; l != NULL; l = l->next)
     {
       GdkDeviceWinpointer *device = GDK_DEVICE_WINPOINTER (l->data);
 
@@ -911,7 +929,7 @@ winpointer_find_system_device_in_device_manager (POINTER_DEVICE_INFO *info)
 }
 
 static void
-winpointer_enumerate_devices (void)
+winpointer_enumerate_devices (GdkDeviceManagerWin32 *device_manager)
 {
   POINTER_DEVICE_INFO *infos = NULL;
   UINT32 infos_count = 0;
@@ -921,7 +939,7 @@ winpointer_enumerate_devices (void)
   do
     {
       infos = g_new0 (POINTER_DEVICE_INFO, infos_count);
-      if (!getPointerDevices (&infos_count, infos))
+      if (!WINPOINTER_API (device_manager, getPointerDevices) (&infos_count, infos))
         {
           WIN32_API_FAILED ("GetPointerDevices");
           g_free (infos);
@@ -930,7 +948,7 @@ winpointer_enumerate_devices (void)
     }
   while (infos_count > 0 && !infos);
 
-  current = _gdk_device_manager->winpointer_devices;
+  current = device_manager->winpointer_devices;
 
   while (current != NULL)
     {
@@ -941,7 +959,7 @@ winpointer_enumerate_devices (void)
         {
           GdkSeat *seat = gdk_device_get_seat (GDK_DEVICE (device));
 
-          _gdk_device_manager->winpointer_devices = g_list_delete_link (_gdk_device_manager->winpointer_devices,
+          device_manager->winpointer_devices = g_list_delete_link (device_manager->winpointer_devices,
                                                                         current);
 
           gdk_device_update_tool (GDK_DEVICE (device), NULL);
@@ -953,7 +971,7 @@ winpointer_enumerate_devices (void)
             gdk_seat_default_remove_tool (GDK_SEAT_DEFAULT (seat), device->tool_eraser);
 
           _gdk_device_set_associated_device (GDK_DEVICE (device), NULL);
-          _gdk_device_remove_physical_device (_gdk_device_manager->core_pointer, GDK_DEVICE (device));
+          _gdk_device_remove_physical_device (device_manager->core_pointer, GDK_DEVICE (device));
 
           gdk_seat_default_remove_physical_device (GDK_SEAT_DEFAULT (seat), GDK_DEVICE (device));
 
@@ -961,7 +979,7 @@ winpointer_enumerate_devices (void)
         }
       else
         {
-          winpointer_device_update_scale_factors (device);
+          winpointer_device_update_scale_factors (device, device_manager);
         }
 
       current = next;
@@ -970,9 +988,9 @@ winpointer_enumerate_devices (void)
   /* create new gdk devices */
   for (i = 0; i < infos_count; i++)
     {
-      if (!winpointer_find_system_device_in_device_manager (&infos[i]))
+      if (!winpointer_find_system_device_in_device_manager (device_manager, &infos[i]))
         {
-          winpointer_create_devices (&infos[i]);
+          winpointer_create_devices (device_manager, &infos[i]);
         }
     }
 
@@ -985,20 +1003,31 @@ winpointer_notifications_window_procedure (HWND hWnd,
                                            WPARAM wParam,
                                            LPARAM lParam)
 {
+  GdkDeviceManagerWin32 *device_manager;
+  CREATESTRUCT *cs = NULL;
+
   switch (uMsg)
     {
+    case WM_NCCREATE:
+      cs = (CREATESTRUCT *)lParam;
+      device_manager = cs->lpCreateParams;
+      SetWindowLongPtr (hWnd, GWLP_USERDATA, (LONG_PTR)device_manager);
+      return TRUE;
     case WM_POINTERDEVICECHANGE:
-      winpointer_enumerate_devices ();
+      device_manager = (GdkDeviceManagerWin32 *) GetWindowLongPtr (hWnd, GWLP_USERDATA);
+      winpointer_enumerate_devices (device_manager);
       return 0;
     }
 
   return DefWindowProcW (hWnd, uMsg, wParam, lParam);
 }
 
-static gboolean
-winpointer_notif_window_create (void)
+static HWND
+winpointer_notif_window_create (GdkDeviceManagerWin32 *device_manager)
 {
   WNDCLASSEXW wndclassex;
+  ATOM notifications_window_class;
+  HWND notification_window_hwnd = NULL;
 
   memset (&wndclassex, 0, sizeof (wndclassex));
   wndclassex.cbSize = sizeof (wndclassex);
@@ -1006,100 +1035,111 @@ winpointer_notif_window_create (void)
   wndclassex.lpfnWndProc = winpointer_notifications_window_procedure;
   wndclassex.hInstance = this_module ();
 
-  if ((notifications_window_class = RegisterClassExW (&wndclassex)) == 0)
-    {
-      WIN32_API_FAILED ("RegisterClassExW");
-      return FALSE;
-    }
+  notifications_window_class = RegisterClassExW (&wndclassex);
 
-  if (!(notifications_window_handle = CreateWindowExW (0,
-                                                       (LPCWSTR)(guintptr)notifications_window_class,
-                                                       L"GdkWin32 Winpointer Notifications",
-                                                       0,
-                                                       0, 0, 0, 0,
-                                                       HWND_MESSAGE,
-                                                       NULL,
-                                                       this_module (),
-                                                       NULL)))
+  if (notifications_window_class != 0)
     {
-      WIN32_API_FAILED ("CreateWindowExW");
-      return FALSE;
-    }
+      notification_window_hwnd = CreateWindowExW (0,
+                                                  (LPCWSTR)(guintptr)notifications_window_class,
+                                                  L"GdkWin32 Winpointer Notifications",
+                                                  0,
+                                                  0, 0, 0, 0,
+                                                  HWND_MESSAGE,
+                                                  NULL,
+                                                  this_module (),
+                                                  device_manager);
 
-  return TRUE;
+      if (notification_window_hwnd == NULL)
+        WIN32_API_FAILED ("CreateWindowExW");
+    }
+  else
+    WIN32_API_FAILED ("RegisterClassExW");
+
+  return notification_window_hwnd;
 }
 
 static gboolean
-winpointer_ensure_procedures (void)
+winpointer_ensure_procedures (GdkDeviceManagerWin32 *device_manager)
 {
-  static HMODULE user32_dll = NULL;
+  GdkDeviceManagerWin32WinpointerFuncs *funcs = NULL;
+  static gsize user32_dll_checked = 0;
 
-  if (!user32_dll)
+  if (g_once_init_enter (&user32_dll_checked))
     {
-      user32_dll = LoadLibraryW (L"user32.dll");
-      if (!user32_dll)
-        {
-          WIN32_API_FAILED ("LoadLibraryW");
-          return FALSE;
-        }
+      HMODULE user32_dll = NULL;
 
-      registerPointerDeviceNotifications = (registerPointerDeviceNotifications_t)
-        GetProcAddress (user32_dll, "RegisterPointerDeviceNotifications");
-      getPointerDevices = (getPointerDevices_t)
-        GetProcAddress (user32_dll, "GetPointerDevices");
-      getPointerDeviceCursors = (getPointerDeviceCursors_t)
-        GetProcAddress (user32_dll, "GetPointerDeviceCursors");
-      getPointerDeviceRects = (getPointerDeviceRects_t)
-        GetProcAddress (user32_dll, "GetPointerDeviceRects");
-      getPointerType = (getPointerType_t)
-        GetProcAddress (user32_dll, "GetPointerType");
-      getPointerCursorId = (getPointerCursorId_t)
-        GetProcAddress (user32_dll, "GetPointerCursorId");
-      getPointerPenInfo = (getPointerPenInfo_t)
-        GetProcAddress (user32_dll, "GetPointerPenInfo");
-      getPointerTouchInfo = (getPointerTouchInfo_t)
-        GetProcAddress (user32_dll, "GetPointerTouchInfo");
-      getPointerPenInfoHistory = (getPointerPenInfoHistory_t)
-        GetProcAddress (user32_dll, "GetPointerPenInfoHistory");
-      getPointerTouchInfoHistory = (getPointerTouchInfoHistory_t)
-        GetProcAddress (user32_dll, "GetPointerTouchInfoHistory");
-      setGestureConfig = (setGestureConfig_t)
-        GetProcAddress (user32_dll, "SetGestureConfig");
-      setWindowFeedbackSetting = (setWindowFeedbackSetting_t)
-        GetProcAddress (user32_dll, "SetWindowFeedbackSetting");
+      user32_dll = LoadLibraryW (L"user32.dll");
+      if (user32_dll)
+        {
+          funcs = g_new0 (GdkDeviceManagerWin32WinpointerFuncs, 1);
+
+          funcs->registerPointerDeviceNotifications = (registerPointerDeviceNotifications_t)
+            GetProcAddress (user32_dll, "RegisterPointerDeviceNotifications");
+          funcs->getPointerDevices = (getPointerDevices_t)
+            GetProcAddress (user32_dll, "GetPointerDevices");
+          funcs->getPointerDeviceCursors = (getPointerDeviceCursors_t)
+            GetProcAddress (user32_dll, "GetPointerDeviceCursors");
+          funcs->getPointerDeviceRects = (getPointerDeviceRects_t)
+            GetProcAddress (user32_dll, "GetPointerDeviceRects");
+          funcs->getPointerType = (getPointerType_t)
+            GetProcAddress (user32_dll, "GetPointerType");
+          funcs->getPointerCursorId = (getPointerCursorId_t)
+            GetProcAddress (user32_dll, "GetPointerCursorId");
+          funcs->getPointerPenInfo = (getPointerPenInfo_t)
+            GetProcAddress (user32_dll, "GetPointerPenInfo");
+          funcs->getPointerTouchInfo = (getPointerTouchInfo_t)
+            GetProcAddress (user32_dll, "GetPointerTouchInfo");
+          funcs->getPointerPenInfoHistory = (getPointerPenInfoHistory_t)
+            GetProcAddress (user32_dll, "GetPointerPenInfoHistory");
+          funcs->getPointerTouchInfoHistory = (getPointerTouchInfoHistory_t)
+            GetProcAddress (user32_dll, "GetPointerTouchInfoHistory");
+          funcs->setGestureConfig = (setGestureConfig_t)
+            GetProcAddress (user32_dll, "SetGestureConfig");
+          funcs->setWindowFeedbackSetting = (setWindowFeedbackSetting_t)
+            GetProcAddress (user32_dll, "SetWindowFeedbackSetting");
+
+          if (funcs->registerPointerDeviceNotifications &&
+              funcs->getPointerDevices &&
+              funcs->getPointerDeviceCursors &&
+              funcs->getPointerDeviceRects &&
+              funcs->getPointerType &&
+              funcs->getPointerCursorId &&
+              funcs->getPointerPenInfo &&
+              funcs->getPointerTouchInfo &&
+              funcs->getPointerPenInfoHistory &&
+              funcs->getPointerTouchInfoHistory)
+            device_manager->winpointer_funcs = funcs;
+        }
+      else
+        WIN32_API_FAILED ("LoadLibraryW");
+
+      g_once_init_leave (&user32_dll_checked, 1);
     }
 
-  return registerPointerDeviceNotifications &&
-         getPointerDevices &&
-         getPointerDeviceCursors &&
-         getPointerDeviceRects &&
-         getPointerType &&
-         getPointerCursorId &&
-         getPointerPenInfo &&
-         getPointerTouchInfo &&
-         getPointerPenInfoHistory &&
-         getPointerTouchInfoHistory &&
-         setGestureConfig;
+  return (device_manager->winpointer_funcs != NULL);
 }
 
 gboolean
-gdk_winpointer_initialize (void)
+gdk_winpointer_initialize (GdkDeviceManagerWin32 *device_manager)
 {
-  if (!winpointer_ensure_procedures ())
+  HWND notification_hwnd = NULL;
+  if (!winpointer_ensure_procedures (device_manager))
     return FALSE;
 
-  if (!winpointer_notif_window_create ())
+  notification_hwnd = winpointer_notif_window_create (device_manager);
+  if (notification_hwnd == NULL)
     return FALSE;
 
-  if (!registerPointerDeviceNotifications (notifications_window_handle, FALSE))
+  if (!WINPOINTER_API (device_manager, registerPointerDeviceNotifications) (notification_hwnd, FALSE))
     {
       WIN32_API_FAILED ("RegisterPointerDeviceNotifications");
       return FALSE;
     }
 
-  ignored_interactions = g_ptr_array_new ();
+  device_manager->winpointer_notification_hwnd = notification_hwnd;
+  device_manager->ignored_interactions = g_ptr_array_new ();
 
-  winpointer_enumerate_devices ();
+  winpointer_enumerate_devices (device_manager);
 
   return TRUE;
 }
@@ -1120,13 +1160,14 @@ gdk_winpointer_initialize_surface (GdkSurface *surface)
                         TABLET_DISABLE_FLICKS |
                         TABLET_DISABLE_FLICKFALLBACKKEYS);
 
-  winpointer_ensure_procedures ();
+  GdkDeviceManagerWin32 *device_manager = GDK_WIN32_DISPLAY (gdk_surface_get_display (surface))->device_manager;
+  winpointer_ensure_procedures (device_manager);
 
   key = GlobalAddAtom (MICROSOFT_TABLETPENSERVICE_PROPERTY);
   API_CALL (SetPropW, (hwnd, (LPCWSTR)(guintptr)key, val));
   GlobalDeleteAtom (key);
 
-  if (setGestureConfig != NULL)
+  if (WINPOINTER_API (device_manager, setGestureConfig) != NULL)
     {
       GESTURECONFIG gesture_config;
       memset (&gesture_config, 0, sizeof (gesture_config));
@@ -1135,10 +1176,10 @@ gdk_winpointer_initialize_surface (GdkSurface *surface)
       gesture_config.dwWant = 0;
       gesture_config.dwBlock = GC_ALLGESTURES;
 
-      API_CALL (setGestureConfig, (hwnd, 0, 1, &gesture_config, sizeof (gesture_config)));
+      API_CALL (WINPOINTER_API (device_manager,setGestureConfig), (hwnd, 0, 1, &gesture_config, sizeof (gesture_config)));
     }
 
-  if (setWindowFeedbackSetting != NULL)
+  if (WINPOINTER_API (device_manager, setWindowFeedbackSetting) != NULL)
     {
       FEEDBACK_TYPE feedbacks[] = {
         FEEDBACK_TOUCH_CONTACTVISUALIZATION,
@@ -1159,7 +1200,7 @@ gdk_winpointer_initialize_surface (GdkSurface *surface)
         {
           BOOL setting = FALSE;
 
-          API_CALL (setWindowFeedbackSetting, (hwnd, feedbacks[i], 0, sizeof (BOOL), &setting));
+          API_CALL (WINPOINTER_API (device_manager, setWindowFeedbackSetting), (hwnd, feedbacks[i], 0, sizeof (BOOL), &setting));
         }
     }
 }
