@@ -34,6 +34,7 @@ struct _GskGpuFrameClass
                                                                          const graphene_rect_t  *opaque);
   void                  (* end)                                         (GskGpuFrame            *self,
                                                                          GdkDrawContext         *context);
+  void                  (* sync)                                        (GskGpuFrame            *self);
   GskGpuImage *         (* upload_texture)                              (GskGpuFrame            *self,
                                                                          gboolean                with_mipmap,
                                                                          GdkTexture             *texture);
@@ -53,6 +54,12 @@ struct _GskGpuFrameClass
                                                                          GskGpuBuffer           *vertex_buffer,
                                                                          GskGpuBuffer           *globals_buffer,
                                                                          GskGpuOp               *op);
+  gpointer              (* alloc_op)                                    (GskGpuFrame            *self,
+                                                                         const GskGpuOpClass    *op_class);
+  void                  (* start_node)                                  (GskGpuFrame            *self,
+                                                                         GskRenderNode          *node,
+                                                                         gsize                   pos);
+  void                  (* end_node)                                    (GskGpuFrame            *self);
 };
 
 GType                   gsk_gpu_frame_get_type                          (void) G_GNUC_CONST;
@@ -74,6 +81,7 @@ void                    gsk_gpu_frame_begin                             (GskGpuF
                                                                          const graphene_rect_t  *opaque);
 void                    gsk_gpu_frame_end                               (GskGpuFrame            *self,
                                                                          GdkDrawContext         *context);
+void                    gsk_gpu_frame_sync                              (GskGpuFrame            *self);
 
 GdkDrawContext *        gsk_gpu_frame_get_context                       (GskGpuFrame            *self) G_GNUC_PURE;
 GskGpuDevice *          gsk_gpu_frame_get_device                        (GskGpuFrame            *self) G_GNUC_PURE;
@@ -82,7 +90,7 @@ gboolean                gsk_gpu_frame_should_optimize                   (GskGpuF
                                                                          GskGpuOptimizations     optimization) G_GNUC_PURE;
 
 gpointer                gsk_gpu_frame_alloc_op                          (GskGpuFrame            *self,
-                                                                         gsize                   size);
+                                                                         const GskGpuOpClass    *op_class);
 GskGpuImage *           gsk_gpu_frame_upload_texture                    (GskGpuFrame            *self,
                                                                          gboolean                with_mipmap,
                                                                          GdkTexture             *texture);
@@ -116,12 +124,16 @@ void                    gsk_gpu_frame_render                            (GskGpuF
 gboolean                gsk_gpu_frame_download_texture                  (GskGpuFrame            *self,
                                                                          gint64                  timestamp,
                                                                          GdkTexture             *texture,
-                                                                         GdkMemoryFormat         format,
-                                                                         GdkColorState          *color_state,
                                                                          guchar                 *data,
-                                                                         gsize                   stride);
+                                                                         const GdkMemoryLayout  *layout,
+                                                                         GdkColorState          *color_state);
 GskGpuOp               *gsk_gpu_frame_get_last_op                       (GskGpuFrame            *self);
 
+void                    gsk_gpu_frame_start_node                        (GskGpuFrame            *self,
+                                                                         GskRenderNode          *node,
+                                                                         gsize                   pos);
+void                    gsk_gpu_frame_end_node                          (GskGpuFrame            *self);
+
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GskGpuFrame, g_object_unref)
 
 G_END_DECLS
