@@ -655,7 +655,8 @@ accesskit_role_for_context (GtkATContext *context)
     return ACCESSKIT_ROLE_GENERIC_CONTAINER;
 
   /* ARIA does not have a "password entry" role, so we need to fudge it here */
-  if (GTK_IS_PASSWORD_ENTRY (accessible))
+  if (role == GTK_ACCESSIBLE_ROLE_TEXT_BOX &&
+      gtk_accessible_is_password_text (accessible))
     return ACCESSKIT_ROLE_PASSWORD_INPUT;
 
   /* ARIA does not have a "scroll area" role */
@@ -697,14 +698,23 @@ set_bounds (GtkAccessible *accessible, accesskit_node *node)
       accesskit_rect bounds = {0, 0, width, height};
 
       if (GTK_IS_ROOT (accessible) && GTK_IS_NATIVE (accessible))
-        gtk_native_get_surface_transform (GTK_NATIVE (accessible), &p.x, &p.y);
+        {
+          GdkSurface *surface = gtk_native_get_surface (GTK_NATIVE (accessible));
+          accesskit_affine scale = accesskit_affine_scale (gdk_surface_get_scale (surface));
+          accesskit_affine translate;
+
+          gtk_native_get_surface_transform (GTK_NATIVE (accessible), &p.x, &p.y);
+          translate = accesskit_affine_translate (p);
+
+          transform = accesskit_affine_mul (scale, translate);
+        }
       else
         {
           p.x = x;
           p.y = y;
+          transform = accesskit_affine_translate (p);
         }
 
-      transform = accesskit_affine_translate (p);
       accesskit_node_set_transform (node, transform);
       accesskit_node_set_bounds (node, bounds);
     }
