@@ -26,6 +26,7 @@ struct _GtkCssSection
   int                 ref_count;
   GtkCssSection      *parent;
   GFile              *file;
+  GBytes             *bytes;
   GtkCssLocation      start_location;
   GtkCssLocation      end_location;   /* end location if parser is %NULL */
 };
@@ -48,6 +49,30 @@ GtkCssSection *
 gtk_css_section_new (GFile                *file,
                      const GtkCssLocation *start,
                      const GtkCssLocation *end)
+{
+  return gtk_css_section_new_with_bytes (file, NULL,start, end);
+}
+
+/**
+ * gtk_css_section_new_with_bytes: (constructor)
+ * @file: (nullable) (transfer none): The file this section refers to
+ * @bytes: (nullable) (transfer none): The bytes this sections refers to
+ * @start: The start location
+ * @end: The end location
+ *
+ * Creates a new `GtkCssSection` referring to the section
+ * in the given `file` or the given `bytes` from the `start` location to the
+ * `end` location.
+ *
+ * Returns: (transfer full): a new `GtkCssSection`
+ *
+ * Since: 4.16
+ **/
+GtkCssSection *
+gtk_css_section_new_with_bytes (GFile  *file,
+                                GBytes *bytes,
+                                const GtkCssLocation *start,
+                                const GtkCssLocation *end)
 {
   GtkCssSection *result;
 
@@ -60,6 +85,8 @@ gtk_css_section_new (GFile                *file,
   result->ref_count = 1;
   if (file)
     result->file = g_object_ref (file);
+  if (bytes)
+    result->bytes = g_bytes_ref (bytes);
   result->start_location = *start;
   result->end_location = *end;
 
@@ -104,6 +131,8 @@ gtk_css_section_unref (GtkCssSection *section)
     gtk_css_section_unref (section->parent);
   if (section->file)
     g_object_unref (section->file);
+  if (section->bytes)
+    g_bytes_unref (section->bytes);
 
   g_free (section);
 }
@@ -115,7 +144,7 @@ gtk_css_section_unref (GtkCssSection *section)
  * Gets the parent section for the given `section`.
  *
  * The parent section is the section that contains this `section`. A special
- * case are sections of  type `GTK_CSS_SECTION_DOCUMEN`T. Their parent will
+ * case are sections of  type `GTK_CSS_SECTION_DOCUMENT`. Their parent will
  * either be `NULL` if they are the original CSS document that was loaded by
  * [method@Gtk.CssProvider.load_from_file] or a section of type
  * `GTK_CSS_SECTION_IMPORT` if it was loaded with an `@import` rule from
@@ -151,6 +180,25 @@ gtk_css_section_get_file (const GtkCssSection *section)
   return section->file;
 }
 
+/**
+ * gtk_css_section_get_bytes:
+ * @section: the section
+ *
+ * Gets the bytes that @section was parsed from.
+ *
+ * Returns: (transfer none) (nullable): the `GBytes` from which the `section`
+ *   was parsed
+ *
+ * Since: 4.16
+ **/
+GBytes *
+gtk_css_section_get_bytes (const GtkCssSection *section)
+{
+  g_return_val_if_fail (section != NULL, NULL);
+
+  return section->bytes;
+}
+
 /**
  * gtk_css_section_get_start_location:
  * @section: the section
@@ -220,7 +268,7 @@ gtk_css_section_print (const GtkCssSection  *section,
       g_string_append (string, "<data>");
     }
 
-  g_string_append_printf (string, ":%zu:%zu", 
+  g_string_append_printf (string, ":%zu:%zu",
                           section->start_location.lines + 1,
                           section->start_location.line_chars + 1);
   if (section->start_location.lines != section->end_location.lines ||
