@@ -16,95 +16,57 @@
 
 #include "config.h"
 
-#include <gdk/gdk.h>
+#include <gsk/gsk.h>
 #include "gdktextureutilsprivate.h"
 #include "gtkscalerprivate.h"
 #include "gtksnapshot.h"
+#include "gtk/gtksvg.h"
+#include "gtk/gtksymbolicpaintable.h"
 
 #include "gdk/gdktextureprivate.h"
 #include "gdk/loaders/gdkpngprivate.h"
 #include "gdk/gdkdebugprivate.h"
 #include "gtk/gtkdebug.h"
 #include "gtk/gtkenums.h"
-
-#include <librsvg/rsvg.h>
+#include "gtk/gtkbitmaskprivate.h"
 
 /* {{{ svg helpers */
 
-static gboolean
-gdk_texture_get_rsvg_handle_size (RsvgHandle *handle, gdouble *out_width, gdouble *out_height)
-{
-#if LIBRSVG_CHECK_VERSION (2,52,0)
-  return rsvg_handle_get_intrinsic_size_in_pixels (handle, out_width, out_height);
-#else
-  RsvgDimensionData dim;
-  rsvg_handle_get_dimensions (handle, &dim);
-  if (out_width)
-    *out_width = dim.width;
-  if (out_height)
-    *out_height = dim.height;
-  return TRUE;
-#endif
-}
-
 static GdkTexture *
-gdk_texture_new_from_rsvg (RsvgHandle  *handle,
-                           int          width,
-                           int          height,
-                           GError     **error)
+svg_to_texture (GtkSvg  *svg,
+                int      width,
+                int      height,
+                GdkRGBA *colors,
+                size_t   n_colors)
 {
-  int stride;
-  guchar *data;
+  GtkSnapshot *snapshot;
+  GskRenderNode *node;
   cairo_surface_t *surface;
   cairo_t *cr;
-  GdkTexture *texture = NULL;
+  GdkTexture *texture;
 
-  stride = width * 4;
-  data = g_new0 (guchar, stride * height);
+  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, width, height);
 
-  surface = cairo_image_surface_create_for_data (data,
-                                                 CAIRO_FORMAT_ARGB32,
-                                                 width, height,
-                                                 stride);
+  snapshot = gtk_snapshot_new ();
 
-  cr = cairo_create (surface);
+  gtk_symbolic_paintable_snapshot_symbolic (GTK_SYMBOLIC_PAINTABLE (svg),
+                                            snapshot,
+                                            width, height,
+                                            colors, n_colors);
 
-#if !LIBRSVG_CHECK_VERSION (2,52,0)
-  {
-    RsvgDimensionData dim;
-    gdouble sx,sy,s;
+  node = gtk_snapshot_free_to_node (snapshot);
 
-    rsvg_handle_get_dimensions (handle, &dim);
-    sx = (gdouble)width / dim.width;
-    sy = (gdouble)height / dim.height;
-    s = MIN (sx, sy);
-
-    cairo_scale (cr, s, s);
-  }
-
-  if (!rsvg_handle_render_cairo (handle, cr))
-    g_set_error (error, GDK_TEXTURE_ERROR, GDK_TEXTURE_ERROR_CORRUPT_IMAGE,
-                 "Error rendering SVG document (%s)", cairo_status_to_string (cairo_status (cr)));
-  else
-#else
-  if (rsvg_handle_render_document (handle, cr,
-                                   &(RsvgRectangle) { 0, 0, width, height },
-                                   error))
-#endif
+  if (node)
     {
-      GBytes *bytes;
+      cr = cairo_create (surface);
+      gsk_render_node_draw (node, cr);
+      cairo_destroy (cr);
 
-      bytes = g_bytes_new (data, stride * height);
-      texture = gdk_memory_texture_new (width, height,
-                                        GDK_MEMORY_DEFAULT,
-                                        bytes,
-                                        stride);
-      g_bytes_unref (bytes);
+      gsk_render_node_unref (node);
     }
 
-  cairo_destroy (cr);
+  texture = gdk_texture_new_for_surface (surface);
   cairo_surface_destroy (surface);
-  g_free (data);
 
   return texture;
 }
@@ -112,78 +74,41 @@ gdk_texture_new_from_rsvg (RsvgHandle  *handle,
 static GdkTexture *
 gdk_texture_new_from_svg_bytes (GBytes  *bytes,
                                 double   scale,
+                                GdkRGBA *colors,
+                                size_t   n_colors,
                                 GError **error)
 {
-  const guchar *data;
-  gsize len;
-  RsvgHandle *handle;
+  GtkSvg *svg;
   GdkTexture *texture;
   int width, height;
-  double w, h;
 
-  data = g_bytes_get_data (bytes, &len);
-  handle = rsvg_handle_new_from_data (data, len, error);
-  if (!handle)
-    return NULL;
+  svg = gtk_svg_new_from_bytes (bytes);
 
-  if (!gdk_texture_get_rsvg_handle_size (handle, &w, &h))
+  width = gdk_paintable_get_intrinsic_width (GDK_PAINTABLE (svg));
+  height = gdk_paintable_get_intrinsic_width (GDK_PAINTABLE (svg));
+
+  if (width == 0 || height == 0)
     {
       g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
                            "Svg image has no intrinsic size; please set one");
-      g_object_unref (handle);
+      g_object_unref (svg);
       return NULL;
     }
 
-  width = ceil (w * scale);
-  height = ceil (h * scale);
-
-  texture = gdk_texture_new_from_rsvg (handle, width, height, error);
-  g_object_unref (handle);
+  texture = svg_to_texture (svg,
+                            ceil (scale * width),
+                            ceil (scale * height),
+                            colors, n_colors);
+  g_object_unref (svg);
 
   return texture;
 }
 
 /* }}} */
- /* {{{ Symbolic processing */
-
-static char *
-make_stylesheet (const char *fg_color,
-                 const char *success_color,
-                 const char *warning_color,
-                 const char *error_color)
-{
-  return g_strconcat ("rect,circle,path,.foreground-fill {\n"
-                      " fill:", fg_color, "!important;\n"
-                      "}\n"
-                      ".warning,.warning-fill {\n"
-                      " fill:", warning_color, "!important;\n"
-                      "}\n"
-                      ".error,.error-fill {\n"
-                      "  fill:", error_color, "!important;\n"
-                      "}\n"
-                      ".success,.success-fill {\n"
-                      "  fill:", success_color, "!important;\n"
-                      "}\n"
-                      ".transparent-fill {\n"
-                      "  fill: none !important;\n"
-                      "}\n"
-                      ".foreground-stroke {\n"
-                      "  stroke:", fg_color, "!important;\n"
-                      "}\n"
-                      ".warning-stroke {\n"
-                      "  stroke:", warning_color, "!important;\n"
-                      "}\n"
-                      ".error-stroke {\n"
-                      "  stroke:", error_color, "!important;\n"
-                      "}\n"
-                      ".success-stroke {\n"
-                      "  stroke:", success_color, "!important;\n"
-                      "}",
-                      NULL);
-}
+/* {{{ Symbolic processing */
 
 static GdkTexture *
-load_symbolic_svg (RsvgHandle  *handle,
+load_symbolic_svg (GtkSvg      *svg,
                    int          width,
                    int          height,
                    const char  *fg_color,
@@ -192,23 +117,14 @@ load_symbolic_svg (RsvgHandle  *handle,
                    const char  *error_color,
                    GError     **error)
 {
-  GdkTexture *texture = NULL;
-  char *stylesheet;
+  GdkRGBA colors[4];
 
-  stylesheet = make_stylesheet (fg_color, success_color, warning_color, error_color);
+  gdk_rgba_parse (&colors[GTK_SYMBOLIC_COLOR_FOREGROUND], fg_color);
+  gdk_rgba_parse (&colors[GTK_SYMBOLIC_COLOR_SUCCESS], success_color);
+  gdk_rgba_parse (&colors[GTK_SYMBOLIC_COLOR_WARNING], warning_color);
+  gdk_rgba_parse (&colors[GTK_SYMBOLIC_COLOR_ERROR], error_color);
 
-  if (!rsvg_handle_set_stylesheet (handle, (const guint8 *) stylesheet, strlen (stylesheet), error))
-    {
-      g_prefix_error (error, "Could not set stylesheet");
-      goto out;
-    }
-
-  texture = gdk_texture_new_from_rsvg (handle, width, height, error);
-
-out:
-  g_free (stylesheet);
-
-  return texture;
+  return svg_to_texture (svg, width, height, colors, 4);
 }
 
 static gboolean
@@ -325,8 +241,7 @@ gdk_texture_new_from_bytes_symbolic (GBytes    *bytes,
                                      GError   **error)
 
 {
-  RsvgHandle *handle;
-  double w, h;
+  GtkSvg *svg;
   const char *r_string = "rgb(255,0,0)";
   const char *g_string = "rgb(0,255,0)";
   gboolean only_fg;
@@ -334,28 +249,24 @@ gdk_texture_new_from_bytes_symbolic (GBytes    *bytes,
   GBytes *data_bytes;
   GdkTexture *texture;
 
-  handle = rsvg_handle_new_from_data (g_bytes_get_data (bytes, NULL),
-                                      g_bytes_get_size (bytes),
-                                      error);
+  svg = gtk_svg_new_from_bytes (bytes);
 
-  if (width == 0 || height == 0)
+  if (width == 0 && height == 0)
     {
-      /* Fetch size from the original icon */
-      if (!gdk_texture_get_rsvg_handle_size (handle, &w, &h))
+      width = gdk_paintable_get_intrinsic_width (GDK_PAINTABLE (svg));
+      height = gdk_paintable_get_intrinsic_height (GDK_PAINTABLE (svg));
+      if (width == 0 || height == 0)
         {
           g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
                        "Symbolic icon has no intrinsic size; please set one in its SVG");
-          g_object_unref (handle);
+          g_object_unref (svg);
           return NULL;
         }
-
-      width = (int) ceil (w);
-      height = (int) ceil (h);
     }
 
   if (!svg_has_symbolic_classes (bytes))
     {
-      texture = gdk_texture_new_from_rsvg (handle, width, height, error);
+      texture = svg_to_texture (svg, width, height, NULL, 0);
 
       if (texture)
         texture = keep_alpha (texture);
@@ -363,7 +274,7 @@ gdk_texture_new_from_bytes_symbolic (GBytes    *bytes,
       if (out_only_fg)
         *out_only_fg = TRUE;
 
-      g_object_unref (handle);
+      g_object_unref (svg);
 
       return texture;
     }
@@ -390,7 +301,7 @@ gdk_texture_new_from_bytes_symbolic (GBytes    *bytes,
        * channels, with the color of the fg being implicitly
        * the "rest", as all color fractions should add up to 1.
        */
-      loaded = load_symbolic_svg (handle, width, height,
+      loaded = load_symbolic_svg (svg, width, height,
                                   g_string,
                                   plane == 0 ? r_string : g_string,
                                   plane == 1 ? r_string : g_string,
@@ -419,7 +330,7 @@ gdk_texture_new_from_bytes_symbolic (GBytes    *bytes,
 
 out:
   g_bytes_unref (data_bytes);
-  g_object_unref (handle);
+  g_object_unref (svg);
 
   if (out_only_fg)
     *out_only_fg = only_fg;
@@ -540,18 +451,22 @@ gdk_texture_new_from_stream_at_scale (GInputStream  *stream,
                                       GCancellable  *cancellable,
                                       GError       **error)
 {
-  RsvgHandle *handle;
+  GBytes *bytes;
+  GtkSvg *svg;
   GdkTexture *texture;
 
   if (only_fg)
     *only_fg = FALSE;
 
-  handle = rsvg_handle_new_from_stream_sync (stream, NULL, RSVG_HANDLE_FLAGS_NONE, NULL, error);
-  if (!handle)
+  bytes = input_stream_get_bytes (stream, error);
+  if (!bytes)
     return NULL;
 
-  texture = gdk_texture_new_from_rsvg (handle, width, height, error);
-  g_object_unref (handle);
+  svg = gtk_svg_new_from_bytes (bytes);
+  texture = svg_to_texture (svg, width, height, NULL, 0);
+
+  g_object_unref (svg);
+  g_bytes_unref (bytes);
 
   return texture;
 }
@@ -684,10 +599,25 @@ gdk_paintable_new_from_bytes_scaled (GBytes *bytes,
       /* We know these formats can't be scaled */
       return GDK_PAINTABLE (gdk_texture_new_from_bytes (bytes, NULL));
     }
+  else if (scale == 1)
+    {
+      return GDK_PAINTABLE (gdk_texture_new_from_svg_bytes (bytes, scale, NULL, 0, NULL));
+    }
   else
     {
-      return GDK_PAINTABLE (gdk_texture_new_from_svg_bytes (bytes, scale, NULL));
+      GdkTexture *texture;
+      GdkPaintable *paintable;
+
+      texture = gdk_texture_new_from_svg_bytes (bytes, scale, NULL, 0, NULL);
+      if (texture)
+        {
+          paintable = gtk_scaler_new (GDK_PAINTABLE (texture), scale);
+          g_object_unref (texture);
+          return paintable;
+        }
     }
+
+  return NULL;
 }
 
 GdkPaintable *
@@ -776,14 +706,16 @@ set_missing_attribute_error (GError     **error,
                "Missing attribute: %s", name);
 }
 
-static void
+static gboolean
 markup_filter_attributes (const char *element_name,
                           const char **attribute_names,
                           const char **attribute_values,
+                          GError     **error,
                           const char  *name,
                           ...)
 {
   va_list ap;
+  guint64 filtered = 0;
 
   va_start (ap, name);
   while (name)
@@ -792,12 +724,39 @@ markup_filter_attributes (const char *element_name,
 
       ptr = va_arg (ap, const char **);
 
-      *ptr = NULL;
+      if (ptr)
+        *ptr = NULL;
+
+      /* Ignoring a whole namespace */
+      if (g_str_has_suffix (name, ":*"))
+        {
+          GPatternSpec *spec = g_pattern_spec_new (name);
+
+          for (int i = 0; attribute_names[i]; i++)
+            {
+              if (filtered & (1ull << i))
+                continue;
+
+              if (g_pattern_spec_match_string (spec, attribute_names[i]))
+                {
+                  filtered |= 1ull << i;
+                  continue;
+                }
+            }
+
+          g_pattern_spec_free (spec);
+        }
+
       for (int i = 0; attribute_names[i]; i++)
         {
+          if (filtered & (1ull << i))
+            continue;
+
           if (strcmp (attribute_names[i], name) == 0)
             {
-              *ptr = attribute_values[i];
+              if (ptr)
+                *ptr = attribute_values[i];
+              filtered |= 1ull << i;
               break;
             }
         }
@@ -806,6 +765,20 @@ markup_filter_attributes (const char *element_name,
     }
 
   va_end (ap);
+
+  for (guint i = 0; attribute_names[i]; i++)
+    {
+      if ((filtered & (1ull << i)) == 0)
+        {
+          g_set_error (error, G_MARKUP_ERROR,
+                       G_MARKUP_ERROR_UNKNOWN_ATTRIBUTE,
+                       "attribute '%s' is invalid for element '%s'",
+                       attribute_names[i], element_name);
+          return FALSE;
+        }
+    }
+
+  return TRUE;
 }
 
 static GskPath *
@@ -883,6 +856,7 @@ start_element_cb (GMarkupParseContext  *context,
       markup_filter_attributes (element_name,
                                 attribute_names,
                                 attribute_values,
+                                NULL,
                                 "width", &width_attr,
                                 "height", &height_attr,
                                 NULL);
@@ -919,7 +893,12 @@ start_element_cb (GMarkupParseContext  *context,
       /* Done */
       return;
     }
-  else if (strcmp (element_name, "g") == 0)
+  else if (strcmp (element_name, "g") == 0 ||
+           strcmp (element_name, "defs") == 0 ||
+           strcmp (element_name, "style") == 0 ||
+           g_str_has_prefix (element_name, "sodipodi:") ||
+           g_str_has_prefix (element_name, "inkscape:"))
+
     {
       /* Do nothing */
       return;
@@ -936,6 +915,7 @@ start_element_cb (GMarkupParseContext  *context,
       markup_filter_attributes (element_name,
                                 attribute_names,
                                 attribute_values,
+                                NULL,
                                 "cx", &cx_attr,
                                 "cy", &cy_attr,
                                 "r", &r_attr,
@@ -994,6 +974,7 @@ start_element_cb (GMarkupParseContext  *context,
       markup_filter_attributes (element_name,
                                 attribute_names,
                                 attribute_values,
+                                NULL,
                                 "x", &x_attr,
                                 "y", &y_attr,
                                 "width", &width_attr,
@@ -1071,6 +1052,7 @@ start_element_cb (GMarkupParseContext  *context,
       markup_filter_attributes (element_name,
                                 attribute_names,
                                 attribute_values,
+                                NULL,
                                 "d", &path_attr,
                                 NULL);
 
@@ -1096,34 +1078,33 @@ start_element_cb (GMarkupParseContext  *context,
 
   g_assert (path != NULL);
 
-  if (!g_markup_collect_attributes (element_name,
-                                    attribute_names,
-                                    attribute_values,
-                                    error,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "class", &class_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "opacity", &opacity_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "fill-rule", &fill_rule_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "fill-opacity", &fill_opacity_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "stroke-width", &stroke_width_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "stroke-opacity", &stroke_opacity_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "stroke-linecap", &stroke_linecap_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "stroke-linejoin", &stroke_linejoin_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "stroke-miterlimit", &stroke_miterlimit_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "stroke-dasharray", &stroke_dasharray_attr,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "stroke-dashoffset", &stroke_dashoffset_attr,
-                                    /* The following attributes are explicitly ignored */
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "fill", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "stroke", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "style", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "id", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "color", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "overflow", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "d", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "cx", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "cy", NULL,
-                                    G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "r", NULL,
-                                     /* Other attributes cause an error, and we'll fall back to loading a texture */
-                                    G_MARKUP_COLLECT_INVALID))
+  if (!markup_filter_attributes (element_name,
+                                 attribute_names,
+                                 attribute_values,
+                                 error,
+                                 "class", &class_attr,
+                                 "opacity", &opacity_attr,
+                                 "fill-rule", &fill_rule_attr,
+                                 "fill-opacity", &fill_opacity_attr,
+                                 "stroke-width", &stroke_width_attr,
+                                 "stroke-opacity", &stroke_opacity_attr,
+                                 "stroke-linecap", &stroke_linecap_attr,
+                                 "stroke-linejoin", &stroke_linejoin_attr,
+                                 "stroke-miterlimit", &stroke_miterlimit_attr,
+                                 "stroke-dasharray", &stroke_dasharray_attr,
+                                 "stroke-dashoffset", &stroke_dashoffset_attr,
+                                 "fill", NULL,
+                                 "stroke", NULL,
+                                 "style", NULL,
+                                 "id", NULL,
+                                 "color", NULL,
+                                 "overflow", NULL,
+                                 "d", NULL,
+                                 "cx", NULL,
+                                 "cy", NULL,
+                                 "r", NULL,
+                                 "gpa:*", NULL, /* Ignore gtk namespace attrs */
+                                 NULL))
     goto cleanup;
 
   fill_opacity = 1;
@@ -1246,7 +1227,9 @@ start_element_cb (GMarkupParseContext  *context,
   else
     fill_rule = GSK_FILL_RULE_WINDING;
 
-  stroke = gsk_stroke_new (1);
+  stroke = gsk_stroke_new (2);
+  gsk_stroke_set_line_cap (stroke, GSK_LINE_CAP_ROUND);
+  gsk_stroke_set_line_join (stroke, GSK_LINE_JOIN_ROUND);
 
   if (stroke_width_attr)
     {
@@ -1508,16 +1491,57 @@ gsk_render_node_new_from_filename_symbolic (const char *filename,
 /* }}} */
 /* {{{ Render node recoloring */
 
+static GskStroke *
+apply_weight (const GskStroke *orig,
+              float            weight)
+{
+  GskStroke *stroke = gsk_stroke_copy (orig);
+  float min_width, width, max_width;
+
+  width = gsk_stroke_get_line_width (orig);
+  min_width = width * 0.25;
+  max_width = width * 1.5;
+
+  if (weight < 1.f)
+    {
+      g_assert_not_reached ();
+    }
+  else if (weight < 400.f)
+    {
+      float f = (400.f - weight) / (400.f - 1.f);
+      width = min_width * f + width * (1.f - f);
+    }
+  else if (weight == 400.f)
+    {
+      /* nothing to do */
+    }
+  else if (weight <= 1000.f)
+    {
+      float f = (weight - 400.f) / (1000.f - 400.f);
+      width = max_width * f + width * (1.f - f);
+    }
+  else
+    {
+      g_assert_not_reached ();
+    }
+
+  gsk_stroke_set_line_width (stroke, width);
+
+  return stroke;
+}
+
 static gboolean
 recolor_node (GskRenderNode *node,
-              const GdkRGBA  colors[4],
+              const GdkRGBA *colors,
+              gsize          n_colors,
+              float          weight,
               GtkSnapshot   *snapshot)
 {
   switch ((int) gsk_render_node_get_node_type (node))
     {
     case GSK_CONTAINER_NODE:
       for (guint i = 0; i < gsk_container_node_get_n_children (node); i++)
-        if (!recolor_node (gsk_container_node_get_child (node, i), colors, snapshot))
+        if (!recolor_node (gsk_container_node_get_child (node, i), colors, n_colors, weight, snapshot))
           return FALSE;
       return TRUE;
 
@@ -1527,7 +1551,7 @@ recolor_node (GskRenderNode *node,
 
         gtk_snapshot_save (snapshot);
         gtk_snapshot_transform (snapshot, gsk_transform_node_get_transform (node));
-        ret = recolor_node (gsk_transform_node_get_child (node), colors, snapshot);
+        ret = recolor_node (gsk_transform_node_get_child (node), colors, n_colors, weight, snapshot);
         gtk_snapshot_restore (snapshot);
 
         return ret;
@@ -1538,7 +1562,7 @@ recolor_node (GskRenderNode *node,
         gboolean ret;
 
         gtk_snapshot_push_clip (snapshot, gsk_clip_node_get_clip (node));
-        ret = recolor_node (gsk_clip_node_get_child (node), colors, snapshot);
+        ret = recolor_node (gsk_clip_node_get_child (node), colors, n_colors, weight, snapshot);
         gtk_snapshot_pop (snapshot);
 
         return ret;
@@ -1549,7 +1573,7 @@ recolor_node (GskRenderNode *node,
         gboolean ret;
 
         gtk_snapshot_push_opacity (snapshot, gsk_opacity_node_get_opacity (node));
-        ret = recolor_node (gsk_opacity_node_get_child (node), colors, snapshot);
+        ret = recolor_node (gsk_opacity_node_get_child (node), colors, n_colors, weight, snapshot);
         gtk_snapshot_pop (snapshot);
 
         return ret;
@@ -1562,7 +1586,7 @@ recolor_node (GskRenderNode *node,
         gtk_snapshot_push_fill (snapshot,
                                 gsk_fill_node_get_path (node),
                                 gsk_fill_node_get_fill_rule (node));
-        ret = recolor_node (gsk_fill_node_get_child (node), colors, snapshot);
+        ret = recolor_node (gsk_fill_node_get_child (node), colors, n_colors, weight, snapshot);
         gtk_snapshot_pop (snapshot);
 
         return ret;
@@ -1572,11 +1596,15 @@ recolor_node (GskRenderNode *node,
     case GSK_STROKE_NODE:
       {
         gboolean ret;
+        GskStroke *stroke;
 
+        stroke = apply_weight (gsk_stroke_node_get_stroke (node), weight);
         gtk_snapshot_push_stroke (snapshot,
                                   gsk_stroke_node_get_path (node),
-                                  gsk_stroke_node_get_stroke (node));
-        ret = recolor_node (gsk_stroke_node_get_child (node), colors, snapshot);
+                                  stroke);
+        gsk_stroke_free (stroke);
+
+        ret = recolor_node (gsk_stroke_node_get_child (node), colors, n_colors, weight, snapshot);
         gtk_snapshot_pop (snapshot);
 
         return ret;
@@ -1619,6 +1647,7 @@ gboolean
 gsk_render_node_recolor (GskRenderNode  *node,
                          const GdkRGBA  *colors,
                          gsize           n_colors,
+                         float           weight,
                          GskRenderNode **recolored)
 {
   GtkSnapshot *snapshot;
@@ -1631,7 +1660,7 @@ gsk_render_node_recolor (GskRenderNode  *node,
     }
 
   snapshot = gtk_snapshot_new ();
-  ret = recolor_node (node, colors, snapshot);
+  ret = recolor_node (node, colors, n_colors, weight, snapshot);
   *recolored = gtk_snapshot_free_to_node (snapshot);
 
   if (!ret)
