@@ -22,10 +22,14 @@
 
 #include "gtkapplicationwindow.h"
 #include "gtkwindowprivate.h"
+#include "gtkapplicationwindow.h"
 
 #include "gtkactionmuxerprivate.h"
 #include "gtkapplicationaccelsprivate.h"
 
+#include "gtkenumsprivate.h"
+#include "gtkprivatetypebuiltins.h"
+
 G_BEGIN_DECLS
 
 void                    gtk_application_window_set_id                   (GtkApplicationWindow     *window,
@@ -47,6 +51,9 @@ GtkApplicationAccels *  gtk_application_get_application_accels          (GtkAppl
 void                    gtk_application_set_screensaver_active          (GtkApplication           *application,
                                                                          gboolean                  active);
 
+void             gtk_application_save                            (GtkApplication       *application);
+void             gtk_application_forget                          (GtkApplication       *application);
+
 #define GTK_TYPE_APPLICATION_IMPL                           (gtk_application_impl_get_type ())
 #define GTK_APPLICATION_IMPL_CLASS(class)                   (G_TYPE_CHECK_CLASS_CAST ((class),                     \
                                                              GTK_TYPE_APPLICATION_IMPL,                            \
@@ -67,16 +74,19 @@ typedef struct
   GObjectClass parent_class;
 
   void        (* startup)                   (GtkApplicationImpl          *impl,
-                                             gboolean                     register_session);
+                                             gboolean                     support_save);
   void        (* shutdown)                  (GtkApplicationImpl          *impl);
 
   void        (* before_emit)               (GtkApplicationImpl          *impl,
                                              GVariant                    *platform_data);
 
   void        (* window_added)              (GtkApplicationImpl          *impl,
-                                             GtkWindow                   *window);
+                                             GtkWindow                   *window,
+                                             GVariant                    *state);
   void        (* window_removed)            (GtkApplicationImpl          *impl,
                                              GtkWindow                   *window);
+  void        (* window_forget)             (GtkApplicationImpl          *impl,
+                                             GtkWindow                   *window);
   void        (* active_window_changed)     (GtkApplicationImpl          *impl,
                                              GtkWindow                   *window);
   void        (* handle_window_realize)     (GtkApplicationImpl          *impl,
@@ -98,9 +108,22 @@ typedef struct
   gboolean    (* is_inhibited)              (GtkApplicationImpl          *impl,
                                              GtkApplicationInhibitFlags   flags);
 
-  gboolean    (* prefers_app_menu)          (GtkApplicationImpl          *impl);
+  GtkRestoreReason
+               (* get_restore_reason)       (GtkApplicationImpl          *impl);
 
+  void         (* clear_restore_reason)     (GtkApplicationImpl          *impl);
 
+  void         (* collect_global_state)     (GtkApplicationImpl          *impl,
+                                             GVariantBuilder             *state);
+  void         (* restore_global_state)     (GtkApplicationImpl          *impl,
+                                             GVariant                    *state);
+  void         (* collect_window_state)     (GtkApplicationImpl          *impl,
+                                             GtkWindow                   *window,
+                                             GVariantBuilder             *state);
+  void         (* store_state)              (GtkApplicationImpl          *impl,
+                                             GVariant                    *state);
+  void         (* forget_state)             (GtkApplicationImpl          *impl);
+  GVariant *   (* retrieve_state)           (GtkApplicationImpl          *impl);
 } GtkApplicationImplClass;
 
 #define GTK_TYPE_APPLICATION_IMPL_DBUS                      (gtk_application_impl_dbus_get_type ())
@@ -128,17 +151,14 @@ typedef struct
   char            *menubar_path;
   guint            menubar_id;
 
-  /* Session management... */
-  GDBusProxy      *sm_proxy;
-  GDBusProxy      *client_proxy;
-  char            *client_path;
-  GDBusProxy      *ss_proxy;
+  char            *instance_id;
+  GtkRestoreReason reason;
 
   /* Portal support */
   GDBusProxy      *inhibit_proxy;
-  GSList *inhibit_handles;
+  GSList          *inhibit_handles;
   guint            state_changed_handler;
-  char *           session_id;
+  char            *session_path;
   guint            session_state;
 } GtkApplicationImplDBus;
 
@@ -146,9 +166,6 @@ typedef struct
 {
   GtkApplicationImplClass parent_class;
 
-  /* returns floating */
-  GVariant *  (* get_window_system_id)      (GtkApplicationImplDBus      *dbus,
-                                             GtkWindow                   *window);
 } GtkApplicationImplDBusClass;
 
 GType                   gtk_application_impl_get_type                   (void);
@@ -157,18 +174,22 @@ GType                   gtk_application_impl_x11_get_type               (void);
 GType                   gtk_application_impl_wayland_get_type           (void);
 GType                   gtk_application_impl_quartz_get_type            (void);
 GType                   gtk_application_impl_android_get_type           (void);
+GType                   gtk_application_impl_win32_get_type             (void);
 
 GtkApplicationImpl *    gtk_application_impl_new                        (GtkApplication              *application,
                                                                          GdkDisplay                  *display);
 void                    gtk_application_impl_startup                    (GtkApplicationImpl          *impl,
-                                                                         gboolean                     register_session);
+                                                                         gboolean                     support_save);
 void                    gtk_application_impl_shutdown                   (GtkApplicationImpl          *impl);
 void                    gtk_application_impl_before_emit                (GtkApplicationImpl          *impl,
                                                                          GVariant                    *platform_data);
 void                    gtk_application_impl_window_added               (GtkApplicationImpl          *impl,
-                                                                         GtkWindow                   *window);
+                                                                         GtkWindow                   *window,
+                                                                         GVariant                    *state);
 void                    gtk_application_impl_window_removed             (GtkApplicationImpl          *impl,
                                                                          GtkWindow                   *window);
+void                    gtk_application_impl_window_forget              (GtkApplicationImpl          *impl,
+                                                                         GtkWindow                   *window);
 void                    gtk_application_impl_active_window_changed      (GtkApplicationImpl          *impl,
                                                                          GtkWindow                   *window);
 void                    gtk_application_impl_handle_window_realize      (GtkApplicationImpl          *impl,
@@ -190,11 +211,30 @@ gboolean                gtk_application_impl_is_inhibited               (GtkAppl
 
 char *                 gtk_application_impl_dbus_get_window_path       (GtkApplicationImplDBus      *dbus,
                                                                          GtkWindow                   *window);
-gboolean                gtk_application_impl_prefers_app_menu           (GtkApplicationImpl          *impl);
-
 
 void                    gtk_application_impl_quartz_setup_menu          (GMenuModel                  *model,
                                                                          GtkActionMuxer              *muxer);
 
+GtkRestoreReason        gtk_application_impl_get_restore_reason         (GtkApplicationImpl          *impl);
+
+void                    gtk_application_impl_clear_restore_reason       (GtkApplicationImpl          *impl);
+
+void                    gtk_application_impl_collect_global_state       (GtkApplicationImpl          *impl,
+                                                                         GVariantBuilder             *builder);
+void                    gtk_application_impl_restore_global_state       (GtkApplicationImpl          *impl,
+                                                                         GVariant                    *state);
+
+void                    gtk_application_impl_collect_window_state       (GtkApplicationImpl          *impl,
+                                                                         GtkWindow                   *window,
+                                                                         GVariantBuilder             *builder);
+
+void                    gtk_application_impl_store_state                (GtkApplicationImpl          *impl,
+                                                                         GVariant                    *state);
+void                    gtk_application_impl_forget_state               (GtkApplicationImpl          *impl);
+GVariant *              gtk_application_impl_retrieve_state             (GtkApplicationImpl          *impl);
+
+GVariant *              gtk_application_impl_dbus_get_window_state      (GtkApplicationImplDBus *dbus,
+                                                                         GtkWindow              *window);
+
 G_END_DECLS
 
