@@ -19,7 +19,7 @@
 
 #include "config.h"
 
-#include "gtkapplicationwindow.h"
+#include "gtkapplicationwindowprivate.h"
 
 #include "gtkapplicationprivate.h"
 #include "gtkwidgetprivate.h"
@@ -28,6 +28,8 @@
 #include "gtksettings.h"
 #include "deprecated/gtkshortcutswindowprivate.h"
 #include "gtktooltipprivate.h"
+#include "gtkprivate.h"
+#include "gtktypebuiltins.h"
 
 #include <glib/gi18n-lib.h>
 
@@ -430,7 +432,7 @@ gtk_application_window_measure (GtkWidget      *widget,
           int menubar_height = 0;
 
           gtk_widget_measure (priv->menubar, GTK_ORIENTATION_VERTICAL,
-                              for_size, &menubar_height, NULL, NULL, NULL);
+                              -1, &menubar_height, NULL, NULL, NULL);
 
 
           GTK_WIDGET_CLASS (gtk_application_window_parent_class)->measure (widget,
@@ -480,10 +482,12 @@ gtk_application_window_real_size_allocate (GtkWidget *widget,
                           &menubar_height, NULL, NULL, NULL);
 
       menubar_allocation.height = menubar_height;
-      gtk_widget_size_allocate  (priv->menubar, &menubar_allocation, baseline);
+      gtk_widget_size_allocate  (priv->menubar, &menubar_allocation, -1);
 
       child_allocation.y += menubar_height;
       child_allocation.height -= menubar_height;
+      if (baseline != -1)
+        baseline -= child_allocation.y;
       child = gtk_window_get_child (GTK_WINDOW (window));
       if (child != NULL && gtk_widget_get_visible (child))
         gtk_widget_size_allocate (child, &child_allocation, baseline);
@@ -700,6 +704,7 @@ gtk_application_window_class_init (GtkApplicationWindowClass *class)
   gtk_application_window_properties[PROP_SHOW_MENUBAR] =
     g_param_spec_boolean ("show-menubar", NULL, NULL,
                           FALSE, G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_install_properties (object_class, N_PROPS, gtk_application_window_properties);
 }
 
@@ -872,3 +877,19 @@ gtk_application_window_get_help_overlay (GtkApplicationWindow *window)
 
   return priv->help_overlay;
 }
+
+/*< private >
+ * gtk_application_window_save:
+ * @window: a `GtkApplicationWindow`
+ * @state: a `GVariantDict` to add state to
+ *
+ * Save the state of @window and its children to a `GVariant`.
+ *
+ * See [signal@Gtk.ApplicationWindow::save-state] for how to override
+ * what state is saved.
+ */
+void
+gtk_application_window_save (GtkApplicationWindow *window,
+                             GVariantDict         *state)
+{
+}
