@@ -21,8 +21,7 @@
 /**
  * GtkATContext:
  *
- * `GtkATContext` is an abstract class provided by GTK to communicate to
- * platform-specific assistive technologies API.
+ * Communicates with platform-specific assistive technologies API.
  *
  * Each platform supported by GTK implements a `GtkATContext` subclass, and
  * is responsible for updating the accessible state in response to state
@@ -406,7 +405,7 @@ static const char *state_attrs[] = {
   [GTK_ACCESSIBLE_STATE_INVALID]        = "invalid",
   [GTK_ACCESSIBLE_STATE_PRESSED]        = "pressed",
   [GTK_ACCESSIBLE_STATE_SELECTED]       = "selected",
-  [GTK_ACCESSIBLE_STATE_VISITED] = "visited",
+  [GTK_ACCESSIBLE_STATE_VISITED]        = "visited",
 };
 
 /*< private >
@@ -1021,21 +1020,26 @@ gtk_at_context_get_accessible_property (GtkATContext          *self,
 
 static void
 append_to_accessible_relation (GtkATContext          *self,
-                                              GtkAccessibleRelation  relation,
-                                              GtkAccessible         *accessible)
+                               GtkAccessibleRelation  relation,
+                               GtkAccessible         *accessible)
 {
   g_return_if_fail (GTK_IS_AT_CONTEXT (self));
-  GtkAccessibleValue * target_value;
+  GtkAccessibleValue *target_value;
 
   if (gtk_accessible_attribute_set_contains (self->relations, relation))
-    target_value = gtk_accessible_attribute_set_get_value (self->relations, relation);
+    {
+      target_value = gtk_accessible_value_ref (gtk_accessible_attribute_set_get_value (self->relations, relation));
+    }
   else
     {
       target_value = gtk_reference_list_accessible_value_new (NULL);
       gtk_accessible_attribute_set_add (self->relations, relation, target_value);
     }
+
   gtk_reference_list_accessible_value_append (target_value, accessible);
 
+  gtk_accessible_value_unref (target_value);
+
   self->updated_relations |= (1 << relation);
 }
 
@@ -1094,6 +1098,8 @@ update_reverse_relation (GtkATContext *self, GtkAccessibleRelation relation, Gtk
                   for (l = gtk_reference_list_accessible_value_get (val); l; l = l->next)
                     {
                       related_context = gtk_accessible_get_at_context (l->data);
+                      if (!related_context)
+                        continue;
                       remove_from_accessible_relation (related_context, reverse_rels_map[i].reverse_rel, self->accessible);
                       g_clear_object (&related_context);
                     }
@@ -1126,10 +1132,13 @@ gtk_at_context_set_accessible_relation (GtkATContext          *self,
 
   gboolean res = FALSE;
 
-  /* We update the reverse relation first, so we can still access
-  * the current value if we're removing it.
+  /* We are setting the relation to a new value,
+  * so we must get rid of the reverse relations first.
   */
-  update_reverse_relation (self, relation, value);
+  update_reverse_relation (self, relation, NULL);
+  /* Now, we can create the new reverse relations if it makes sense. */
+  if (value != NULL)
+    update_reverse_relation (self, relation, value);
 
   if (value != NULL)
     res = gtk_accessible_attribute_set_add (self->relations, relation, value);
@@ -1196,7 +1205,7 @@ static guint8 naming[] = {
   [GTK_ACCESSIBLE_ROLE_ALERT_DIALOG] = NAME_FROM_AUTHOR|GTK_ACCESSIBLE_NAME_REQUIRED,
   [GTK_ACCESSIBLE_ROLE_APPLICATION] = NAME_FROM_AUTHOR|GTK_ACCESSIBLE_NAME_REQUIRED,
   [GTK_ACCESSIBLE_ROLE_ARTICLE] = NAME_FROM_AUTHOR,
-  [GTK_ACCESSIBLE_ROLE_BANNER] = GTK_ACCESSIBLE_NAME_PROHIBITED,
+  [GTK_ACCESSIBLE_ROLE_BANNER] = NAME_FROM_AUTHOR,
   [GTK_ACCESSIBLE_ROLE_BLOCK_QUOTE] = NAME_FROM_AUTHOR,
   [GTK_ACCESSIBLE_ROLE_BUTTON] = NAME_FROM_AUTHOR|NAME_FROM_CONTENT|GTK_ACCESSIBLE_NAME_REQUIRED,
   [GTK_ACCESSIBLE_ROLE_CAPTION] = GTK_ACCESSIBLE_NAME_PROHIBITED,
@@ -1258,7 +1267,7 @@ static guint8 naming[] = {
   [GTK_ACCESSIBLE_ROLE_SEPARATOR] = NAME_FROM_AUTHOR,
   [GTK_ACCESSIBLE_ROLE_SLIDER] = NAME_FROM_AUTHOR|GTK_ACCESSIBLE_NAME_REQUIRED,
   [GTK_ACCESSIBLE_ROLE_SPIN_BUTTON] = NAME_FROM_AUTHOR|GTK_ACCESSIBLE_NAME_REQUIRED,
-  [GTK_ACCESSIBLE_ROLE_STATUS] = GTK_ACCESSIBLE_NAME_PROHIBITED,
+  [GTK_ACCESSIBLE_ROLE_STATUS] = NAME_FROM_AUTHOR,
   [GTK_ACCESSIBLE_ROLE_STRUCTURE] = GTK_ACCESSIBLE_NAME_PROHIBITED,
   [GTK_ACCESSIBLE_ROLE_SWITCH] = NAME_FROM_AUTHOR|NAME_FROM_CONTENT|GTK_ACCESSIBLE_NAME_REQUIRED,
   [GTK_ACCESSIBLE_ROLE_TAB] = NAME_FROM_AUTHOR|NAME_FROM_CONTENT|GTK_ACCESSIBLE_NAME_REQUIRED,
@@ -1267,11 +1276,11 @@ static guint8 naming[] = {
   [GTK_ACCESSIBLE_ROLE_TAB_PANEL] = NAME_FROM_AUTHOR|GTK_ACCESSIBLE_NAME_REQUIRED,
   [GTK_ACCESSIBLE_ROLE_TERMINAL] = NAME_FROM_AUTHOR|GTK_ACCESSIBLE_NAME_REQUIRED,
   [GTK_ACCESSIBLE_ROLE_TEXT_BOX] = NAME_FROM_AUTHOR|GTK_ACCESSIBLE_NAME_REQUIRED,
-  [GTK_ACCESSIBLE_ROLE_TIME] = GTK_ACCESSIBLE_NAME_PROHIBITED,
+  [GTK_ACCESSIBLE_ROLE_TIME] = NAME_FROM_AUTHOR,
   [GTK_ACCESSIBLE_ROLE_TIMER] = NAME_FROM_AUTHOR,
   [GTK_ACCESSIBLE_ROLE_TOGGLE_BUTTON] = NAME_FROM_AUTHOR|GTK_ACCESSIBLE_NAME_REQUIRED,
   [GTK_ACCESSIBLE_ROLE_TOOLBAR] = NAME_FROM_AUTHOR|GTK_ACCESSIBLE_NAME_RECOMMENDED,
-  [GTK_ACCESSIBLE_ROLE_TOOLTIP] = NAME_FROM_AUTHOR|NAME_FROM_CONTENT,
+  [GTK_ACCESSIBLE_ROLE_TOOLTIP] = NAME_FROM_AUTHOR|NAME_FROM_CONTENT| GTK_ACCESSIBLE_NAME_REQUIRED,
   [GTK_ACCESSIBLE_ROLE_TREE] = NAME_FROM_AUTHOR|GTK_ACCESSIBLE_NAME_REQUIRED,
   [GTK_ACCESSIBLE_ROLE_TREE_GRID] = NAME_FROM_AUTHOR|GTK_ACCESSIBLE_NAME_REQUIRED,
   [GTK_ACCESSIBLE_ROLE_TREE_ITEM] = NAME_FROM_AUTHOR|NAME_FROM_CONTENT|GTK_ACCESSIBLE_NAME_REQUIRED,
