@@ -17,8 +17,7 @@
  * Authors: Benjamin Otte <otte@gnome.org>
  */
 
-#ifndef __GTK_CSS_DYNAMIC_PRIVATE_H__
-#define __GTK_CSS_DYNAMIC_PRIVATE_H__
+#pragma once
 
 #include "gtkstyleanimationprivate.h"
 
@@ -34,4 +33,3 @@ GtkStyleAnimation *     gtk_css_dynamic_new             (gint64 timestamp);
 
 G_END_DECLS
 
-#endif /* __GTK_CSS_DYNAMIC_PRIVATE_H__ */
