@@ -38,7 +38,7 @@
 /**
  * GtkDropTarget:
  *
- * `GtkDropTarget` is an event controller to receive Drag-and-Drop operations.
+ * An event controller to receive Drag-and-Drop operations.
  *
  * The most basic way to use a `GtkDropTarget` to receive drops on a
  * widget is to create it via [ctor@Gtk.DropTarget.new], passing in the
@@ -182,7 +182,7 @@ gtk_drop_target_end_drop (GtkDropTarget *self)
 
   if (self->dropping)
     {
-      gdk_drop_finish (self->drop, 0);
+      gdk_drop_finish (self->drop, GDK_ACTION_NONE);
       self->dropping = FALSE;
     }
 
@@ -220,7 +220,7 @@ make_action_unique (GdkDragAction actions)
   if (actions & GDK_ACTION_LINK)
     return GDK_ACTION_LINK;
 
-  return 0;
+  return GDK_ACTION_NONE;
 }
 
 static void
@@ -233,10 +233,17 @@ gtk_drop_target_do_drop (GtkDropTarget *self)
 
   g_signal_emit (self, signals[DROP], 0, &self->value, self->coords.x, self->coords.y, &success);
 
-  if (success)
-    gdk_drop_finish (self->drop, make_action_unique (self->actions & gdk_drop_get_actions (self->drop)));
-  else
-    gdk_drop_finish (self->drop, 0);
+  if (self->drop)
+    {
+      GdkDragAction action;
+
+      if (success)
+        action = make_action_unique (self->actions & gdk_drop_get_actions (self->drop));
+      else
+        action = GDK_ACTION_NONE;
+
+      gdk_drop_finish (self->drop, action);
+    }
 
   self->dropping = FALSE;
 
@@ -358,7 +365,7 @@ static gboolean
 gtk_drop_target_accept (GtkDropTarget *self,
                         GdkDrop       *drop)
 {
-  if ((gdk_drop_get_actions (drop) & gtk_drop_target_get_actions (self)) == 0)
+  if ((gdk_drop_get_actions (drop) & gtk_drop_target_get_actions (self)) == GDK_ACTION_NONE)
     return FALSE;
 
   if (self->formats == NULL)
@@ -441,7 +448,7 @@ gtk_drop_target_handle_event (GtkEventController *controller,
                         G_OBJECT_TYPE_NAME (widget), widget);
             preferred = make_action_unique (preferred);
           }
-        if (preferred &&
+        if (preferred && self->drop &&
             gtk_drop_status (self->drop, self->actions, preferred))
           {
             gtk_widget_set_state_flags (widget, GTK_STATE_FLAG_DROP_ACTIVE, FALSE);
@@ -520,7 +527,7 @@ gtk_drop_target_handle_crossing (GtkEventController    *controller,
           preferred = make_action_unique (preferred);
         }
 
-      if (preferred &&
+      if (preferred && self->drop &&
           gtk_drop_status (self->drop, self->actions, preferred))
         {
           gtk_widget_set_state_flags (widget, GTK_STATE_FLAG_DROP_ACTIVE, FALSE);
@@ -533,7 +540,8 @@ gtk_drop_target_handle_crossing (GtkEventController    *controller,
   else
     {
       if (crossing->new_descendent != NULL ||
-          crossing->new_target == widget)
+          crossing->new_target == widget ||
+          self->drop == NULL)
         return;
 
       g_signal_emit (self, signals[LEAVE], 0);
@@ -544,6 +552,16 @@ gtk_drop_target_handle_crossing (GtkEventController    *controller,
     }
 }
 
+static void
+gtk_drop_target_reset (GtkEventController *controller)
+{
+  GtkDropTarget *self = GTK_DROP_TARGET (controller);
+
+  if (self->drop)
+    g_signal_emit (self, signals[LEAVE], 0);
+  gtk_drop_target_end_drop (self);
+}
+
 static void
 gtk_drop_target_finalize (GObject *object)
 {
@@ -635,6 +653,7 @@ gtk_drop_target_class_init (GtkDropTargetClass *class)
   controller_class->handle_event = gtk_drop_target_handle_event;
   controller_class->filter_event = gtk_drop_target_filter_event;
   controller_class->handle_crossing = gtk_drop_target_handle_crossing;
+  controller_class->reset = gtk_drop_target_reset;
 
   class->accept = gtk_drop_target_accept;
   class->enter = gtk_drop_target_enter;
@@ -642,17 +661,17 @@ gtk_drop_target_class_init (GtkDropTargetClass *class)
   class->drop = gtk_drop_target_drop;
 
   /**
-   * GtkDropTarget:actions: (attributes org.gtk.Property.get=gtk_drop_target_get_actions org.gtk.Property.set=gtk_drop_target_set_actions)
+   * GtkDropTarget:actions:
    *
    * The `GdkDragActions` that this drop target supports.
    */
   properties[PROP_ACTIONS] =
        g_param_spec_flags ("actions", NULL, NULL,
-                           GDK_TYPE_DRAG_ACTION, 0,
+                           GDK_TYPE_DRAG_ACTION, GDK_ACTION_NONE,
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkDropTarget:drop: (attributes org.gtk.Property.get=gtk_drop_target_get_drop) (skip)
+   * GtkDropTarget:drop:
    *
    * The `GdkDrop` that is currently being performed.
    *
@@ -664,7 +683,7 @@ gtk_drop_target_class_init (GtkDropTargetClass *class)
                             GTK_PARAM_READABLE | G_PARAM_DEPRECATED);
 
   /**
-   * GtkDropTarget:current-drop: (attributes org.gtk.Property.get=gtk_drop_target_get_current_drop)
+   * GtkDropTarget:current-drop:
    *
    * The `GdkDrop` that is currently being performed.
    *
@@ -676,7 +695,7 @@ gtk_drop_target_class_init (GtkDropTargetClass *class)
                             GTK_PARAM_READABLE);
 
   /**
-   * GtkDropTarget:formats: (attributes org.gtk.Property.get=gtk_drop_target_get_formats)
+   * GtkDropTarget:formats:
    *
    * The `GdkContentFormats` that determine the supported data formats.
    */
@@ -686,7 +705,7 @@ gtk_drop_target_class_init (GtkDropTargetClass *class)
                            GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
 
   /**
-   * GtkDropTarget:preload: (attributes org.gtk.Property.get=gtk_drop_target_get_preload org.gtk.Property.set=gtk_drop_target_set_preload)
+   * GtkDropTarget:preload:
    *
    * Whether the drop data should be preloaded when the pointer is only
    * hovering over the widget but has not been released.
@@ -713,7 +732,7 @@ gtk_drop_target_class_init (GtkDropTargetClass *class)
                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkDropTarget:value: (attributes org.gtk.Property.get=gtk_drop_target_get_value)
+   * GtkDropTarget:value:
    *
    * The value for this drop operation.
    *
@@ -779,7 +798,7 @@ gtk_drop_target_class_init (GtkDropTargetClass *class)
    *
    * It can be used to set up custom highlighting.
    *
-   * Returns: Preferred action for this drag operation or 0 if
+   * Returns: Preferred action for this drag operation or `GDK_ACTION_NONE` if
    *   dropping is not supported at the current @x,@y location.
    */
   signals[ENTER] =
@@ -803,7 +822,7 @@ gtk_drop_target_class_init (GtkDropTargetClass *class)
    *
    * Emitted while the pointer is moving over the drop target.
    *
-   * Returns: Preferred action for this drag operation or 0 if
+   * Returns: Preferred action for this drag operation or `GDK_ACTION_NONE` if
    *   dropping is not supported at the current @x,@y location.
    */
   signals[MOTION] =
@@ -911,7 +930,7 @@ gtk_drop_target_new (GType         type,
 }
 
 /**
- * gtk_drop_target_get_formats: (attributes org.gtk.Method.get_property=formats)
+ * gtk_drop_target_get_formats:
  * @self: a `GtkDropTarget`
  *
  * Gets the data formats that this drop target accepts.
@@ -983,7 +1002,7 @@ gtk_drop_target_get_gtypes (GtkDropTarget *self,
 }
 
 /**
- * gtk_drop_target_set_actions: (attributes org.gtk.Method.set_property=actions)
+ * gtk_drop_target_set_actions:
  * @self: a `GtkDropTarget`
  * @actions: the supported actions
  *
@@ -1004,7 +1023,7 @@ gtk_drop_target_set_actions (GtkDropTarget *self,
 }
 
 /**
- * gtk_drop_target_get_actions: (attributes org.gtk.Method.get_property=actions)
+ * gtk_drop_target_get_actions:
  * @self: a `GtkDropTarget`
  *
  * Gets the actions that this drop target supports.
@@ -1014,13 +1033,13 @@ gtk_drop_target_set_actions (GtkDropTarget *self,
 GdkDragAction
 gtk_drop_target_get_actions (GtkDropTarget *self)
 {
-  g_return_val_if_fail (GTK_IS_DROP_TARGET (self), 0);
+  g_return_val_if_fail (GTK_IS_DROP_TARGET (self), GDK_ACTION_NONE);
 
   return self->actions;
 }
 
 /**
- * gtk_drop_target_set_preload: (attributes org.gtk.Method.set_property=preload)
+ * gtk_drop_target_set_preload:
  * @self: a `GtkDropTarget`
  * @preload: %TRUE to preload drop data
  *
@@ -1041,7 +1060,7 @@ gtk_drop_target_set_preload (GtkDropTarget *self,
 }
 
 /**
- * gtk_drop_target_get_preload: (attributes org.gtk.Method.get_property=preload)
+ * gtk_drop_target_get_preload:
  * @self: a `GtkDropTarget`
  *
  * Gets whether data should be preloaded on hover.
@@ -1057,7 +1076,7 @@ gtk_drop_target_get_preload (GtkDropTarget *self)
 }
 
 /**
- * gtk_drop_target_get_drop: (attributes org.gtk.Method.get_property=drop)
+ * gtk_drop_target_get_drop:
  * @self: a `GtkDropTarget`
  *
  * Gets the currently handled drop operation.
@@ -1077,7 +1096,7 @@ gtk_drop_target_get_drop (GtkDropTarget *self)
 }
 
 /**
- * gtk_drop_target_get_current_drop: (attributes org.gtk.Method.get_property=current-drop)
+ * gtk_drop_target_get_current_drop:
  * @self: a `GtkDropTarget`
  *
  * Gets the currently handled drop operation.
@@ -1097,7 +1116,7 @@ gtk_drop_target_get_current_drop (GtkDropTarget *self)
 }
 
 /**
- * gtk_drop_target_get_value: (attributes org.gtk.Method.get_property=value)
+ * gtk_drop_target_get_value:
  * @self: a `GtkDropTarget`
  *
  * Gets the current drop data, as a `GValue`.
