@@ -162,7 +162,8 @@ response_cb (GDBusConnection  *connection,
           GtkFileFilter *f = g_list_model_get_item (filters, j);
           if (g_strcmp0 (gtk_file_filter_get_name (f), current_filter_name) == 0)
             {
-              filter_to_select = f;
+              g_set_object (&filter_to_select, f);
+              g_object_unref (f);
               break;
             }
           g_object_unref (f);
@@ -307,7 +308,7 @@ get_filters (GtkFileChooser *self)
   for (i = 0; i < n; i++)
     {
       GtkFileFilter *filter = g_list_model_get_item (filters, i);
-      g_variant_builder_add (&builder, "@(sa(us))", gtk_file_filter_to_gvariant (filter));
+      g_variant_builder_add_value (&builder, gtk_file_filter_to_gvariant (filter));
       g_object_unref (filter);
     }
   g_object_unref (filters);
@@ -346,8 +347,7 @@ serialize_choices (GtkFileChooserNative *self)
     {
       GtkFileChooserNativeChoice *choice = l->data;
 
-      g_variant_builder_add (&builder, "@(ssa(ss)s)",
-                             gtk_file_chooser_native_choice_to_variant (choice));
+      g_variant_builder_add_value (&builder, gtk_file_chooser_native_choice_to_variant (choice));
     }
 
   return g_variant_builder_end (&builder);
@@ -420,12 +420,11 @@ show_portal_file_chooser (GtkFileChooserNative *self,
     }
   if (self->current_file)
     {
-      char *path;
+      const char *path;
 
-      path = g_file_get_path (GTK_FILE_CHOOSER_NATIVE (self)->current_file);
+      path = g_file_peek_path (GTK_FILE_CHOOSER_NATIVE (self)->current_file);
       g_variant_builder_add (&opt_builder, "{sv}", "current_file",
                              g_variant_new_bytestring (path));
-      g_free (path);
     }
 
   if (self->choices)
@@ -476,7 +475,6 @@ gtk_file_chooser_native_portal_show (GtkFileChooserNative *self)
   FilechooserPortalData *data;
   GtkWindow *transient_for;
   GDBusConnection *connection;
-  GtkFileChooserAction action;
   const char *method_name;
   GdkDisplay *display;
 
@@ -506,23 +504,19 @@ gtk_file_chooser_native_portal_show (GtkFileChooserNative *self)
       return TRUE;
     }
 
-  action = gtk_file_chooser_get_action (GTK_FILE_CHOOSER (self));
-
-  if (action == GTK_FILE_CHOOSER_ACTION_OPEN)
-    method_name = "OpenFile";
-  else if (action == GTK_FILE_CHOOSER_ACTION_SAVE)
-    method_name = "SaveFile";
-  else if (action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)
-    method_name = "OpenFile";
-  else
+  switch (gtk_file_chooser_get_action (GTK_FILE_CHOOSER (self)))
     {
-      GtkAlertDialog *alert;
-
-      alert = gtk_alert_dialog_new (_("The create-folder action is not supported with portals"));
-      gtk_alert_dialog_show (alert, transient_for);
-      g_object_unref (alert);
-
-      return TRUE;
+    case GTK_FILE_CHOOSER_ACTION_OPEN:
+      method_name = "OpenFile";
+      break;
+    case GTK_FILE_CHOOSER_ACTION_SAVE:
+      method_name = "SaveFile";
+      break;
+    case GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER:
+      method_name = "OpenFile";
+      break;
+    default:
+      g_assert_not_reached ();
     }
 
   data = g_new0 (FilechooserPortalData, 1);
