@@ -20,8 +20,7 @@
 /**
  * GtkGridLayout:
  *
- * `GtkGridLayout` is a layout manager which arranges child widgets in
- * rows and columns.
+ * Arranges child widgets in rows and columns.
  *
  * Children have an "attach point" defined by the horizontal and vertical
  * index of the cell they occupy; children can span multiple rows or columns.
@@ -47,7 +46,6 @@
 
 #include "gtkcsspositionvalueprivate.h"
 #include "gtkdebug.h"
-#include "gtkintl.h"
 #include "gtklayoutchild.h"
 #include "gtkorientable.h"
 #include "gtkprivate.h"
@@ -159,50 +157,42 @@ gtk_grid_layout_child_class_init (GtkGridLayoutChildClass *klass)
   gobject_class->get_property = gtk_grid_layout_child_get_property;
 
   /**
-   * GtkGridLayoutChild:column: (attributes org.gtk.Property.get=gtk_grid_layout_child_get_column org.gtk.Property.set=gtk_grid_layout_child_set_column)
+   * GtkGridLayoutChild:column:
    *
    * The column to place the child in.
    */
   child_props[PROP_CHILD_COLUMN] =
-    g_param_spec_int ("column",
-                      P_("Column"),
-                      P_("The column to place the child in"),
+    g_param_spec_int ("column", NULL, NULL,
                       G_MININT, G_MAXINT, 0,
                       GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkGridLayoutChild:row: (attributes org.gtk.Property.get=gtk_grid_layout_child_get_row org.gtk.Property.set=gtk_grid_layout_child_set_row)
+   * GtkGridLayoutChild:row:
    *
    * The row to place the child in.
    */
   child_props[PROP_CHILD_ROW] =
-    g_param_spec_int ("row",
-                      P_("Row"),
-                      P_("The row to place the child in"),
+    g_param_spec_int ("row", NULL, NULL,
                       G_MININT, G_MAXINT, 0,
                       GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkGridLayoutChild:column-span: (attributes org.gtk.Property.get=gtk_grid_layout_child_get_column_span org.gtk.Property.set=gtk_grid_layout_child_set_column_span)
+   * GtkGridLayoutChild:column-span:
    *
    * The number of columns the child spans to.
    */
   child_props[PROP_CHILD_COLUMN_SPAN] =
-    g_param_spec_int ("column-span",
-                      P_("Column span"),
-                      P_("The number of columns that a child spans"),
+    g_param_spec_int ("column-span", NULL, NULL,
                       1, G_MAXINT, 1,
                       GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkGridLayoutChild:row-span: (attributes org.gtk.Property.get=gtk_grid_layout_child_get_row_span org.gtk.Property.set=gtk_grid_layout_child_set_row_span)
+   * GtkGridLayoutChild:row-span:
    *
    * The number of rows the child spans to.
    */
   child_props[PROP_CHILD_ROW_SPAN] =
-    g_param_spec_int ("row-span",
-                      P_("Row span"),
-                      P_("The number of rows that a child spans"),
+    g_param_spec_int ("row-span", NULL, NULL,
                       1, G_MAXINT, 1,
                       GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
@@ -217,7 +207,7 @@ gtk_grid_layout_child_init (GtkGridLayoutChild *self)
 }
 
 /**
- * gtk_grid_layout_child_set_row: (attributes org.gtk.Method.set_property=row)
+ * gtk_grid_layout_child_set_row:
  * @child: a `GtkGridLayoutChild`
  * @row: the row for @child
  *
@@ -240,7 +230,7 @@ gtk_grid_layout_child_set_row (GtkGridLayoutChild *child,
 }
 
 /**
- * gtk_grid_layout_child_get_row: (attributes org.gtk.Method.get_property=row)
+ * gtk_grid_layout_child_get_row:
  * @child: a `GtkGridLayoutChild`
  *
  * Retrieves the row number to which @child attaches its top side.
@@ -256,7 +246,7 @@ gtk_grid_layout_child_get_row (GtkGridLayoutChild *child)
 }
 
 /**
- * gtk_grid_layout_child_set_column: (attributes org.gtk.Method.set_property=column)
+ * gtk_grid_layout_child_set_column:
  * @child: a `GtkGridLayoutChild`
  * @column: the attach point for @child
  *
@@ -279,7 +269,7 @@ gtk_grid_layout_child_set_column (GtkGridLayoutChild *child,
 }
 
 /**
- * gtk_grid_layout_child_get_column: (attributes org.gtk.Method.get_property=column)
+ * gtk_grid_layout_child_get_column:
  * @child: a `GtkGridLayoutChild`
  *
  * Retrieves the column number to which @child attaches its left side.
@@ -295,7 +285,7 @@ gtk_grid_layout_child_get_column (GtkGridLayoutChild *child)
 }
 
 /**
- * gtk_grid_layout_child_set_column_span: (attributes org.gtk.Method.set_property=column-span)
+ * gtk_grid_layout_child_set_column_span:
  * @child: a `GtkGridLayoutChild`
  * @span: the span of @child
  *
@@ -318,7 +308,7 @@ gtk_grid_layout_child_set_column_span (GtkGridLayoutChild *child,
 }
 
 /**
- * gtk_grid_layout_child_get_column_span: (attributes org.gtk.Method.get_property=column-span)
+ * gtk_grid_layout_child_get_column_span:
  * @child: a `GtkGridLayoutChild`
  *
  * Retrieves the number of columns that @child spans to.
@@ -334,7 +324,7 @@ gtk_grid_layout_child_get_column_span (GtkGridLayoutChild *child)
 }
 
 /**
- * gtk_grid_layout_child_set_row_span: (attributes org.gtk.Method.set_property=row-span)
+ * gtk_grid_layout_child_set_row_span:
  * @child: a `GtkGridLayoutChild`
  * @span: the span of @child
  *
@@ -357,7 +347,7 @@ gtk_grid_layout_child_set_row_span (GtkGridLayoutChild *child,
 }
 
 /**
- * gtk_grid_layout_child_get_row_span: (attributes org.gtk.Method.get_property=row-span)
+ * gtk_grid_layout_child_get_row_span:
  * @child: a `GtkGridLayoutChild`
  *
  * Retrieves the number of rows that @child spans to.
@@ -1171,7 +1161,7 @@ grid_request_allocate (GridRequest    *request,
   lines = &request->lines[orientation];
   spacing = get_spacing (request->layout, request->widget, orientation);
 
-  baseline = gtk_widget_get_allocated_baseline (request->widget);
+  baseline = gtk_widget_get_baseline (request->widget);
 
   if (orientation == GTK_ORIENTATION_VERTICAL && baseline != -1 &&
       self->baseline_row >= lines->min && self->baseline_row < lines->max &&
@@ -1307,7 +1297,7 @@ grid_request_position (GridRequest    *request,
   lines = &request->lines[orientation];
   spacing = get_spacing (request->layout, request->widget, orientation);
 
-  allocated_baseline = gtk_widget_get_allocated_baseline (request->widget);
+  allocated_baseline = gtk_widget_get_baseline (request->widget);
 
   position = 0;
   for (i = 0; i < lines->max - lines->min; i++)
@@ -1472,7 +1462,9 @@ allocate_child (GridRequest        *request,
   attach = &grid_child->attach[orientation];
 
   *position = lines->lines[attach->pos - lines->min].position;
-  if (attach->span == 1 && gtk_widget_get_valign (child) == GTK_ALIGN_BASELINE)
+  if (attach->span == 1 &&
+      (gtk_widget_get_valign (child) == GTK_ALIGN_BASELINE_CENTER ||
+       gtk_widget_get_valign (child) == GTK_ALIGN_BASELINE_FILL))
     *baseline = lines->lines[attach->pos - lines->min].allocated_baseline;
   else
     *baseline = -1;
@@ -1661,63 +1653,53 @@ gtk_grid_layout_class_init (GtkGridLayoutClass *klass)
   gobject_class->finalize = gtk_grid_layout_finalize;
 
   /**
-   * GtkGridLayout:row-spacing: (attributes org.gtk.Property.get=gtk_grid_layout_get_row_spacing org.gtk.Property.set=gtk_grid_layout_set_row_spacing)
+   * GtkGridLayout:row-spacing:
    *
    * The amount of space between to consecutive rows.
    */
   layout_props[PROP_ROW_SPACING] =
-    g_param_spec_int ("row-spacing",
-                      P_("Row spacing"),
-                      P_("The amount of space between two consecutive rows"),
+    g_param_spec_int ("row-spacing", NULL, NULL,
                       0, G_MAXINT16, 0,
                       GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkGridLayout:column-spacing: (attributes org.gtk.Property.get=gtk_grid_layout_get_column_spacing org.gtk.Property.set=gtk_grid_layout_set_column_spacing)
+   * GtkGridLayout:column-spacing:
    *
    * The amount of space between to consecutive columns.
    */
   layout_props[PROP_COLUMN_SPACING] =
-    g_param_spec_int ("column-spacing",
-                      P_("Column spacing"),
-                      P_("The amount of space between two consecutive columns"),
+    g_param_spec_int ("column-spacing", NULL, NULL,
                       0, G_MAXINT16, 0,
                       GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkGridLayout:row-homogeneous: (attributes org.gtk.Property.get=gtk_grid_layout_get_row_homogeneous org.gtk.Property.set=gtk_grid_layout_set_row_homogeneous)
+   * GtkGridLayout:row-homogeneous:
    *
    * Whether all the rows in the grid have the same height.
    */
   layout_props[PROP_ROW_HOMOGENEOUS] =
-    g_param_spec_boolean ("row-homogeneous",
-                          P_("Row Homogeneous"),
-                          P_("If TRUE, the rows are all the same height"),
+    g_param_spec_boolean ("row-homogeneous", NULL, NULL,
                           FALSE,
                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkGridLayout:column-homogeneous: (attributes org.gtk.Property.get=gtk_grid_layout_get_column_homogeneous org.gtk.Property.set=gtk_grid_layout_set_column_homogeneous)
+   * GtkGridLayout:column-homogeneous:
    *
    * Whether all the columns in the grid have the same width.
    */
   layout_props[PROP_COLUMN_HOMOGENEOUS] =
-    g_param_spec_boolean ("column-homogeneous",
-                          P_("Column Homogeneous"),
-                          P_("If TRUE, the columns are all the same width"),
+    g_param_spec_boolean ("column-homogeneous", NULL, NULL,
                           FALSE,
                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkGridLayout:baseline-row: (attributes org.gtk.Property.get=gtk_grid_layout_get_baseline_row org.gtk.Property.set=gtk_grid_layout_set_baseline_row)
+   * GtkGridLayout:baseline-row:
    *
    * The row to align to the baseline, when `GtkWidget:valign` is set
    * to %GTK_ALIGN_BASELINE.
    */
   layout_props[PROP_BASELINE_ROW] =
-    g_param_spec_int ("baseline-row",
-                      P_("Baseline Row"),
-                      P_("The row to align the to the baseline when valign is GTK_ALIGN_BASELINE"),
+    g_param_spec_int ("baseline-row", NULL, NULL,
                       0, G_MAXINT, 0,
                       GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
@@ -1743,7 +1725,7 @@ gtk_grid_layout_new (void)
 }
 
 /**
- * gtk_grid_layout_set_row_homogeneous: (attributes org.gtk.Method.set_property=row-homogeneous)
+ * gtk_grid_layout_set_row_homogeneous:
  * @grid: a `GtkGridLayout`
  * @homogeneous: %TRUE to make rows homogeneous
  *
@@ -1766,7 +1748,7 @@ gtk_grid_layout_set_row_homogeneous (GtkGridLayout *grid,
 }
 
 /**
- * gtk_grid_layout_get_row_homogeneous: (attributes org.gtk.Method.get_property=row-homogeneous)
+ * gtk_grid_layout_get_row_homogeneous:
  * @grid: a `GtkGridLayout`
  *
  * Checks whether all rows of @grid should have the same height.
@@ -1782,7 +1764,7 @@ gtk_grid_layout_get_row_homogeneous (GtkGridLayout *grid)
 }
 
 /**
- * gtk_grid_layout_set_row_spacing: (attributes org.gtk.Method.set_property=row-spacing)
+ * gtk_grid_layout_set_row_spacing:
  * @grid: a `GtkGridLayout`
  * @spacing: the amount of space between rows, in pixels
  *
@@ -1805,7 +1787,7 @@ gtk_grid_layout_set_row_spacing (GtkGridLayout *grid,
 }
 
 /**
- * gtk_grid_layout_get_row_spacing: (attributes org.gtk.Method.get_property=row-spacing)
+ * gtk_grid_layout_get_row_spacing:
  * @grid: a `GtkGridLayout`
  *
  * Retrieves the spacing set with gtk_grid_layout_set_row_spacing().
@@ -1821,7 +1803,7 @@ gtk_grid_layout_get_row_spacing (GtkGridLayout *grid)
 }
 
 /**
- * gtk_grid_layout_set_column_homogeneous: (attributes org.gtk.Method.set_property=column-homogeneous)
+ * gtk_grid_layout_set_column_homogeneous:
  * @grid: a `GtkGridLayout`
  * @homogeneous: %TRUE to make columns homogeneous
  *
@@ -1844,7 +1826,7 @@ gtk_grid_layout_set_column_homogeneous (GtkGridLayout *grid,
 }
 
 /**
- * gtk_grid_layout_get_column_homogeneous: (attributes org.gtk.Method.get_property=column-homogeneous)
+ * gtk_grid_layout_get_column_homogeneous:
  * @grid: a `GtkGridLayout`
  *
  * Checks whether all columns of @grid should have the same width.
@@ -1860,7 +1842,7 @@ gtk_grid_layout_get_column_homogeneous (GtkGridLayout *grid)
 }
 
 /**
- * gtk_grid_layout_set_column_spacing: (attributes org.gtk.Method.set_property=column-spacing)
+ * gtk_grid_layout_set_column_spacing:
  * @grid: a `GtkGridLayout`
  * @spacing: the amount of space between columns, in pixels
  *
@@ -1883,7 +1865,7 @@ gtk_grid_layout_set_column_spacing (GtkGridLayout *grid,
 }
 
 /**
- * gtk_grid_layout_get_column_spacing: (attributes org.gtk.Method.get_property=column-spacing)
+ * gtk_grid_layout_get_column_spacing:
  * @grid: a `GtkGridLayout`
  *
  * Retrieves the spacing set with gtk_grid_layout_set_column_spacing().
@@ -2010,7 +1992,7 @@ gtk_grid_layout_get_row_baseline_position (GtkGridLayout *grid,
 }
 
 /**
- * gtk_grid_layout_set_baseline_row: (attributes org.gtk.Method.set_property=baseline-row)
+ * gtk_grid_layout_set_baseline_row:
  * @grid: a `GtkGridLayout`
  * @row: the row index
  *
@@ -2035,7 +2017,7 @@ gtk_grid_layout_set_baseline_row (GtkGridLayout *grid,
 }
 
 /**
- * gtk_grid_layout_get_baseline_row: (attributes org.gtk.Method.get_property=baseline-row)
+ * gtk_grid_layout_get_baseline_row:
  * @grid: a `GtkGridLayout`
  *
  * Retrieves the row set with gtk_grid_layout_set_baseline_row().
