@@ -1021,6 +1021,8 @@ gtk_image_snapshot (GtkWidget   *widget,
   height = gtk_widget_get_height (widget);
   ratio = gdk_paintable_get_intrinsic_aspect_ratio (GDK_PAINTABLE (image->icon_helper));
 
+  gtk_snapshot_push_isolation (snapshot, GSK_ISOLATION_ALL);
+
   if (ratio == 0)
     {
       gdk_paintable_snapshot (GDK_PAINTABLE (image->icon_helper), snapshot, width, height);
@@ -1046,7 +1048,7 @@ gtk_image_snapshot (GtkWidget   *widget,
       if (baseline == -1)
         y = floor(height - ceil (h)) / 2;
       else
-        y = CLAMP (baseline - h * gtk_image_get_baseline_align (image), 0, height - ceil (h));
+        y = CLAMP (baseline - floor (ceil (h) * gtk_image_get_baseline_align (image)), 0, height - ceil (h));
 
       if (x != 0 || y != 0)
         {
@@ -1060,6 +1062,8 @@ gtk_image_snapshot (GtkWidget   *widget,
           gdk_paintable_snapshot (GDK_PAINTABLE (image->icon_helper), snapshot, w, h);
         }
     }
+
+  gtk_snapshot_pop (snapshot); /* isolation */
 }
 
 static void
