@@ -194,12 +194,14 @@ static const GdkDebugKey gtk_debug_keys[] = {
   { "builder-objects", GTK_DEBUG_BUILDER_OBJECTS, "Log unused GtkBuilder objects" },
   { "no-css-cache", GTK_DEBUG_NO_CSS_CACHE, "Disable style property cache" },
   { "interactive", GTK_DEBUG_INTERACTIVE, "Enable the GTK inspector" },
+  { "touch-ui", GTK_DEBUG_TOUCHSCREEN, "Show touch ui elements for pointer events" },
   { "snapshot", GTK_DEBUG_SNAPSHOT, "Generate debug render nodes" },
   { "accessibility", GTK_DEBUG_A11Y, "Information about accessibility state changes" },
   { "iconfallback", GTK_DEBUG_ICONFALLBACK, "Information about icon fallback" },
   { "invert-text-dir", GTK_DEBUG_INVERT_TEXT_DIR, "Invert the default text direction" },
   { "css", GTK_DEBUG_CSS, "Information about deprecated CSS features" },
   { "builder", GTK_DEBUG_BUILDER, "Information about deprecated GtkBuilder features" },
+  { "session-mgmt", GTK_DEBUG_SESSION, "Information about session saving" },
 };
 
 /* This checks to see if the process is running suid or sgid
@@ -281,7 +283,28 @@ gtk_disable_portals (void)
   if (pre_initialized)
     g_warning ("gtk_disable_portals() must be called before gtk_init()");
 
-  gdk_disable_portals ();
+  gdk_disable_all_portals ();
+}
+
+
+/**
+ * gtk_disable_portal_interfaces:
+ * @portal_interfaces: (array zero-terminated=1) (not nullable):
+ *     a %NULL-terminated array of portal interface names to disable
+ *
+ * Prevents GTK from using the specified portals.
+ *
+ * This should only be used in portal implementations, apps must not call it.
+ *
+ * Since: 4.22
+ */
+void
+gtk_disable_portal_interfaces (const char **portal_interfaces)
+{
+  if (pre_initialized)
+    g_warning ("gtk_disable_portal_interfaces() must be called before gtk_init()");
+
+  gdk_disable_portals (portal_interfaces);
 }
 
 #ifdef G_PLATFORM_WIN32
@@ -1388,10 +1411,7 @@ handle_pointing_event (GdkEvent *event)
   type = gdk_event_get_event_type (event);
   sequence = gdk_event_get_event_sequence (event);
 
-  if (type == GDK_SCROLL &&
-      (gdk_device_get_source (device) == GDK_SOURCE_TOUCHPAD ||
-       gdk_device_get_source (device) == GDK_SOURCE_TRACKPOINT ||
-       gdk_device_get_source (device) == GDK_SOURCE_MOUSE))
+  if (type == GDK_SCROLL && !gdk_event_get_device_tool (event))
     {
       /* A bit of a kludge, resolve target lookups for scrolling devices
        * on the seat pointer.
@@ -1717,6 +1737,7 @@ gtk_main_do_event (GdkEvent *event)
     case GDK_PAD_BUTTON_RELEASE:
     case GDK_PAD_RING:
     case GDK_PAD_STRIP:
+    case GDK_PAD_DIAL:
     case GDK_PAD_GROUP_MODE:
     case GDK_GRAB_BROKEN:
       handled_event = gtk_propagate_event (grab_widget, event);
