@@ -410,6 +410,8 @@ set_button_image_get_info_cb (GObject      *source,
         break;
     };
 
+  g_object_unref (info);
+
 out:
   g_free (data);
 }
@@ -751,6 +753,7 @@ gtk_path_bar_set_file_finish (struct SetFileInfo *info,
       GList *l;
 
       gtk_path_bar_clear_buttons (info->path_bar);
+      g_list_free (info->path_bar->button_list);
       info->path_bar->button_list = g_list_reverse (info->new_buttons);
       info->path_bar->fake_root = info->fake_root;
 
@@ -830,6 +833,7 @@ gtk_path_bar_get_info_callback (GObject      *source,
                                        file_info->root_file,
                                        file_info->first_directory, is_hidden);
   g_clear_object (&file_info->file);
+  g_object_unref (info);
 
   file_info->new_buttons = g_list_prepend (file_info->new_buttons, button_data);
 
