@@ -46,9 +46,12 @@
 /**
  * GtkScale:
  *
- * A `GtkScale` is a slider control used to select a numeric value.
+ * Allows to select a numeric value with a slider control.
  *
- * ![An example GtkScale](scales.png)
+ * <picture>
+ *   <source srcset="scales-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img alt="An example GtkScale" src="scales.png">
+ * </picture>
  *
  * To use it, you’ll probably want to investigate the methods on its base
  * class, [class@Gtk.Range], in addition to the methods for `GtkScale` itself.
@@ -70,6 +73,15 @@
  * the mark. It can be translated with the usual ”translatable” and
  * “context” attributes.
  *
+ * # Shortcuts and Gestures
+ *
+ * `GtkPopoverMenu` supports the following keyboard shortcuts:
+ *
+ * - Arrow keys, <kbd>+</kbd> and <kbd>-</kbd> will increment or decrement
+ *   by step, or by page when combined with <kbd>Ctrl</kbd>.
+ * - <kbd>PgUp</kbd> and <kbd>PgDn</kbd> will increment or decrement by page.
+ * - <kbd>Home</kbd> and <kbd>End</kbd> will set the minimum or maximum value.
+ *
  * # CSS nodes
  *
  * ```
@@ -125,7 +137,7 @@
  *
  * # Accessibility
  *
- * `GtkScale` uses the %GTK_ACCESSIBLE_ROLE_SLIDER role.
+ * `GtkScale` uses the [enum@Gtk.AccessibleRole.slider] role.
  */
 
 
@@ -330,6 +342,8 @@ gtk_scale_allocate_value (GtkScale *scale)
   GtkAllocation value_alloc;
   int range_width, range_height;
   graphene_rect_t slider_bounds;
+  GdkRectangle trough_rect;
+  int slider_center_x, slider_center_y, trough_center_x, trough_center_y;
 
   range_width = gtk_widget_get_width (widget);
   range_height = gtk_widget_get_height (widget);
@@ -338,6 +352,14 @@ gtk_scale_allocate_value (GtkScale *scale)
   if (!gtk_widget_compute_bounds (slider_widget, widget, &slider_bounds))
     graphene_rect_init (&slider_bounds, 0, 0, gtk_widget_get_width (widget), gtk_widget_get_height (widget));
 
+  slider_center_x = slider_bounds.origin.x + slider_bounds.size.width / 2;
+  slider_center_y = slider_bounds.origin.y + slider_bounds.size.height / 2;
+
+  gtk_range_get_range_rect (range, &trough_rect);
+
+  trough_center_x = trough_rect.x + trough_rect.width / 2;
+  trough_center_y = trough_rect.y + trough_rect.height / 2;
+
   gtk_widget_measure (priv->value_widget,
                       GTK_ORIENTATION_HORIZONTAL, -1,
                       &value_alloc.width, NULL,
@@ -353,21 +375,21 @@ gtk_scale_allocate_value (GtkScale *scale)
         {
         case GTK_POS_LEFT:
           value_alloc.x = 0;
-          value_alloc.y = (range_height - value_alloc.height) / 2;
+          value_alloc.y = trough_center_y - value_alloc.height / 2;
           break;
 
         case GTK_POS_RIGHT:
           value_alloc.x = range_width - value_alloc.width;
-          value_alloc.y = (range_height - value_alloc.height) / 2;
+          value_alloc.y = trough_center_y - value_alloc.height / 2;
           break;
 
         case GTK_POS_TOP:
-          value_alloc.x = slider_bounds.origin.x + (slider_bounds.size.width - value_alloc.width) / 2;
-          value_alloc.y = slider_bounds.origin.y - value_alloc.height;
+          value_alloc.x = slider_center_x - value_alloc.width / 2;
+          value_alloc.y = 0;
           break;
 
         case GTK_POS_BOTTOM:
-          value_alloc.x = slider_bounds.origin.x + (slider_bounds.size.width - value_alloc.width) / 2;
+          value_alloc.x = slider_center_x - value_alloc.width / 2;
           value_alloc.y = range_height - value_alloc.height;
           break;
 
@@ -382,21 +404,21 @@ gtk_scale_allocate_value (GtkScale *scale)
         {
         case GTK_POS_LEFT:
           value_alloc.x = 0;
-          value_alloc.y = (slider_bounds.origin.y + (slider_bounds.size.height / 2)) - value_alloc.height / 2;
+          value_alloc.y = slider_center_y - value_alloc.height / 2;
           break;
 
         case GTK_POS_RIGHT:
           value_alloc.x = range_width - value_alloc.width;
-          value_alloc.y = (slider_bounds.origin.y + (slider_bounds.size.height / 2)) - value_alloc.height / 2;
+          value_alloc.y = slider_center_y - value_alloc.height / 2;
           break;
 
         case GTK_POS_TOP:
-          value_alloc.x = (range_width - value_alloc.width) / 2;
+          value_alloc.x = trough_center_x - value_alloc.width / 2;
           value_alloc.y = 0;
           break;
 
         case GTK_POS_BOTTOM:
-          value_alloc.x = (range_width - value_alloc.width) / 2;
+          value_alloc.x = trough_center_x - value_alloc.width / 2;
           value_alloc.y = range_height - value_alloc.height;
           break;
 
@@ -563,7 +585,7 @@ gtk_scale_size_allocate (GtkWidget *widget,
                               &marks_height, NULL,
                               NULL, NULL);
           marks_rect.x = 0;
-          marks_rect.y = 0;
+          marks_rect.y = range_rect.y - marks_height;
           marks_rect.width = range_rect.width;
           marks_rect.height = marks_height;
           gtk_widget_size_allocate (priv->top_marks_widget, &marks_rect, -1);
@@ -669,7 +691,7 @@ gtk_scale_class_init (GtkScaleClass *class)
   class->get_layout_offsets = gtk_scale_real_get_layout_offsets;
 
   /**
-   * GtkScale:digits: (attributes org.gtk.Method.get=gtk_scale_get_digits org.gtk.Method.set=gtk_scale_set_digits)
+   * GtkScale:digits:
    *
    * The number of decimal places that are displayed in the value.
    */
@@ -680,7 +702,7 @@ gtk_scale_class_init (GtkScaleClass *class)
                         GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkScale:draw-value: (attributes org.gtk.Method.get=gtk_scale_get_draw_value org.gtk.Method.set=gtk_scale_set_draw_value)
+   * GtkScale:draw-value:
    *
    * Whether the current value is displayed as a string next to the slider.
    */
@@ -690,7 +712,7 @@ gtk_scale_class_init (GtkScaleClass *class)
                             GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkScale:has-origin: (attributes org.gtk.Method.get=gtk_scale_get_has_origin org.gtk.Method.set=gtk_scale_set_has_origin)
+   * GtkScale:has-origin:
    *
    * Whether the scale has an origin.
    */
@@ -700,7 +722,7 @@ gtk_scale_class_init (GtkScaleClass *class)
                             GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkScale:value-pos: (attributes org.gtk.Method.get=gtk_scale_get_value_pos org.gtk.Method.set=gtk_scale_set_value_pos)
+   * GtkScale:value-pos:
    *
    * The position in which the current value is displayed.
    */
@@ -988,7 +1010,7 @@ gtk_scale_new_with_range (GtkOrientation orientation,
 }
 
 /**
- * gtk_scale_set_digits: (attributes org.gtk.Method.set_property=digits)
+ * gtk_scale_set_digits:
  * @scale: a `GtkScale`
  * @digits: the number of decimal places to display,
  *   e.g. use 1 to display 1.0, 2 to display 1.00, etc
@@ -1035,7 +1057,7 @@ gtk_scale_set_digits (GtkScale *scale,
 }
 
 /**
- * gtk_scale_get_digits: (attributes org.gtk.Method.get_property=digits)
+ * gtk_scale_get_digits:
  * @scale: a `GtkScale`
  *
  * Gets the number of decimal places that are displayed in the value.
@@ -1086,7 +1108,7 @@ update_value_position (GtkScale *scale)
 }
 
 /**
- * gtk_scale_set_draw_value: (attributes org.gtk.Method.set_property=draw-value)
+ * gtk_scale_set_draw_value:
  * @scale: a `GtkScale`
  * @draw_value: %TRUE to draw the value
  *
@@ -1128,7 +1150,7 @@ gtk_scale_set_draw_value (GtkScale *scale,
 }
 
 /**
- * gtk_scale_get_draw_value: (attributes org.gtk.Method.get_property=draw-value)
+ * gtk_scale_get_draw_value:
  * @scale: a `GtkScale`
  *
  * Returns whether the current value is displayed as a string
@@ -1147,7 +1169,7 @@ gtk_scale_get_draw_value (GtkScale *scale)
 }
 
 /**
- * gtk_scale_set_has_origin: (attributes org.gtk.Method.set_property=has-origin)
+ * gtk_scale_set_has_origin:
  * @scale: a `GtkScale`
  * @has_origin: %TRUE if the scale has an origin
  *
@@ -1176,7 +1198,7 @@ gtk_scale_set_has_origin (GtkScale *scale,
 }
 
 /**
- * gtk_scale_get_has_origin: (attributes org.gtk.Method.get_property=has-origin)
+ * gtk_scale_get_has_origin:
  * @scale: a `GtkScale`
  *
  * Returns whether the scale has an origin.
@@ -1192,7 +1214,7 @@ gtk_scale_get_has_origin (GtkScale *scale)
 }
 
 /**
- * gtk_scale_set_value_pos: (attributes org.gtk.Method.set_property=value-pos)
+ * gtk_scale_set_value_pos:
  * @scale: a `GtkScale`
  * @pos: the position in which the current value is displayed
  *
@@ -1218,7 +1240,7 @@ gtk_scale_set_value_pos (GtkScale        *scale,
 }
 
 /**
- * gtk_scale_get_value_pos: (attributes org.gtk.Method.get_property=value-pos)
+ * gtk_scale_get_value_pos:
  * @scale: a `GtkScale`
  *
  * Gets the position in which the current value is displayed.
@@ -1417,11 +1439,12 @@ gtk_scale_measure (GtkWidget      *widget,
   GtkScale *scale = GTK_SCALE (widget);
   GtkScalePrivate *priv = gtk_scale_get_instance_private (scale);
   GtkOrientation scale_orientation;
+  int range_minimum, range_natural, scale_minimum = 0, scale_natural = 0;
 
   GTK_WIDGET_CLASS (gtk_scale_parent_class)->measure (widget,
                                                       orientation,
                                                       for_size,
-                                                      minimum, natural,
+                                                      &range_minimum, &range_natural,
                                                       minimum_baseline, natural_baseline);
 
   scale_orientation = gtk_orientable_get_orientation (GTK_ORIENTABLE (widget));
@@ -1443,8 +1466,8 @@ gtk_scale_measure (GtkWidget      *widget,
 
       marks_size = MAX (top_marks_size, bottom_marks_size);
 
-      *minimum = MAX (*minimum, marks_size);
-      *natural = MAX (*natural, marks_size);
+      scale_minimum = MAX (scale_minimum, marks_size);
+      scale_natural = MAX (scale_natural, marks_size);
     }
 
   if (priv->value_widget)
@@ -1458,29 +1481,32 @@ gtk_scale_measure (GtkWidget      *widget,
         {
           if (orientation == GTK_ORIENTATION_HORIZONTAL)
             {
-              *minimum = MAX (*minimum, min);
-              *natural = MAX (*natural, nat);
+              scale_minimum = MAX (scale_minimum, min);
+              scale_natural = MAX (scale_natural, nat);
             }
           else
             {
-              *minimum += min;
-              *natural += nat;
+              scale_minimum += min;
+              scale_natural += nat;
             }
         }
       else
         {
           if (orientation == GTK_ORIENTATION_HORIZONTAL)
             {
-              *minimum += min;
-              *natural += nat;
+              scale_minimum += min;
+              scale_natural += nat;
             }
           else
             {
-              *minimum = MAX (*minimum, min);
-              *natural = MAX (*natural, nat);
+              scale_minimum = MAX (scale_minimum, min);
+              scale_natural = MAX (scale_natural, nat);
             }
         }
     }
+
+  *minimum = MAX (range_minimum, scale_minimum);
+  *natural = MAX (range_natural, scale_natural);
 }
 
 static void
@@ -1834,7 +1860,7 @@ mark_data_free (MarkData *data)
 {
   g_string_free (data->markup, TRUE);
   g_free (data->context);
-  g_slice_free (MarkData, data);
+  g_free (data);
 }
 
 static void
@@ -1908,7 +1934,7 @@ marks_start_element (GtkBuildableParseContext *context,
           position = g_value_get_enum (&gvalue);
         }
 
-      mark = g_slice_new (MarkData);
+      mark = g_new (MarkData, 1);
       mark->value = value;
       if (position == GTK_POS_LEFT || position == GTK_POS_TOP)
         mark->position = GTK_POS_TOP;
@@ -1968,7 +1994,7 @@ gtk_scale_buildable_custom_tag_start (GtkBuildable       *buildable,
 
   if (strcmp (tagname, "marks") == 0)
     {
-      data = g_slice_new0 (MarksSubparserData);
+      data = g_new0 (MarksSubparserData, 1);
       data->scale = GTK_SCALE (buildable);
       data->builder = builder;
       data->marks = NULL;
@@ -2017,12 +2043,12 @@ gtk_scale_buildable_custom_finished (GtkBuildable *buildable,
         }
 
       g_slist_free (marks_data->marks);
-      g_slice_free (MarksSubparserData, marks_data);
+      g_free (marks_data);
     }
   else
     {
       parent_buildable_iface->custom_finished (buildable, builder, child,
-					       tagname, user_data);
+                                               tagname, user_data);
     }
 
 }
@@ -2030,8 +2056,9 @@ gtk_scale_buildable_custom_finished (GtkBuildable *buildable,
 /**
  * gtk_scale_set_format_value_func:
  * @scale: a `GtkScale`
- * @func: (nullable): function that formats the value
- * @user_data: (closure): user data to pass to @func
+ * @func: (nullable) (scope notified) (closure user_data) (destroy destroy_notify): function
+ *   that formats the value
+ * @user_data: user data to pass to @func
  * @destroy_notify: (nullable): destroy function for @user_data
  *
  * @func allows you to change how the scale value is displayed.
