@@ -161,7 +161,10 @@ deserialize_error_func (const GskParseLocation *start,
       g_string_append_printf (string, "%zu", end->line_chars + 1);
     }
 
-  g_warning ("Error at %s: %s", string->str, error->message);
+  if (error->domain == GTK_CSS_PARSER_WARNING)
+    g_test_message ("Warning at %s: %s", string->str, error->message);
+  else
+    g_warning ("Error at %s: %s", string->str, error->message);
 
   g_string_free (string, TRUE);
 }
@@ -197,6 +200,26 @@ make_random_clip (cairo_rectangle_int_t *int_clip,
                   int                    width,
                   int                    height)
 {
+  const char *clip = g_getenv ("OVERRIDE_CLIP");
+
+  if (clip)
+    {
+      char **str = g_strsplit (clip, ",", -1);
+
+      if (g_strv_length (str) == 4)
+        {
+          int_clip->width = CLAMP (atoi (str[2]), 1, width);
+          int_clip->height = CLAMP (atoi (str[3]), 1, height);
+          int_clip->x = CLAMP (atoi (str[0]), 0, width - int_clip->width);
+          int_clip->y = CLAMP (atoi (str[1]), 0, height - int_clip->height);
+
+          g_strfreev (str);
+          return;
+        }
+
+      g_strfreev (str);
+    }
+
   int_clip->width = g_test_rand_int_range (1, width);
   int_clip->height = g_test_rand_int_range (1, height);
 
@@ -477,6 +500,8 @@ clip_setup (GskRenderNode *node)
   else
     make_random_clip (result, ceil (bounds.size.width), ceil (bounds.size.height));
 
+  g_print ("Node bounds %g %g %g %g\n",
+           bounds.origin.x, bounds.origin.y, bounds.size.width, bounds.size.height);
   g_print ("Random clip rectangle %d %d %d %d\n",
            result->x, result->y, result->width, result->height);
 
@@ -569,6 +594,107 @@ colorflip_create_reference (GskRenderer   *renderer,
   return result;
 }
 
+static gboolean
+rect_shrink_to_int (graphene_rect_t *rect)
+{
+  float x1 = floor (rect->origin.x + rect->size.width);
+  float y1 = floor (rect->origin.y + rect->size.height);
+
+  rect->origin.x = ceil (rect->origin.x);
+  rect->origin.y = ceil (rect->origin.y);
+  rect->size.width = x1 - rect->origin.x;
+  rect->size.height = y1 - rect->origin.y;
+
+  return rect->size.width >=0 && rect->size.height >= 0;
+}
+
+static gpointer
+opaque_setup (GskRenderNode *node)
+{
+  graphene_rect_t opaque;
+
+  if (!gsk_render_node_get_opaque_rect (node, &opaque))
+    {
+      g_test_skip ("Node has no opaque rect");
+      return NULL;
+    }
+
+  if (!rect_shrink_to_int (&opaque))
+    {
+      g_test_skip ("Opaque rect is too small, so it rounds to nothing.");
+      return NULL;
+    }
+
+  return g_memdup2 (&opaque, sizeof (graphene_rect_t));
+}
+
+static GskRenderNode *
+opaque_create_test (GskRenderNode *node,
+                    gconstpointer  data)
+{
+  const graphene_rect_t *opaque = data;
+  graphene_matrix_t matrix;
+  GskRenderNode *clip, *result;
+
+  /* clip to opaque region */
+  clip = gsk_clip_node_new (node, opaque);
+
+  /* Turn image to white, don't touch alpha */
+  graphene_matrix_init_from_float (&matrix,
+                                   (const float []) { 0, 0, 0, 0,
+                                                      0, 0, 0, 0,
+                                                      0, 0, 0, 0,
+                                                      1, 1, 1, 1 });
+  result = gsk_color_matrix_node_new (clip, &matrix, graphene_vec4_zero ());
+
+  gsk_render_node_unref (clip);
+
+  return result;
+}
+
+static GdkTexture *
+opaque_create_reference (GskRenderer   *renderer,
+                         GdkTexture    *texture,
+                         gconstpointer  user_data)
+{
+  const graphene_rect_t *rect = user_data;
+  GBytes *bytes;
+  guchar *data;
+  gsize stride, size;
+  GdkTexture *result;
+
+  stride = ((gsize) rect->size.width + 3) & ~3;
+  size = stride * rect->size.height;
+
+  data = g_malloc (size);
+  memset (data, 0xFF, size);
+  bytes = g_bytes_new_take (data, size);
+
+  result = gdk_memory_texture_new (rect->size.width,
+                                   rect->size.height,
+                                   GDK_MEMORY_A8,
+                                   bytes,
+                                   stride);
+
+  g_bytes_unref (bytes);
+
+  return result;
+}
+
+static GskRenderNode *
+serialize_create_test (GskRenderNode *node,
+                       gconstpointer  data)
+{
+  GskRenderNode *result;
+  GBytes *bytes;
+
+  bytes = gsk_render_node_serialize (node);
+  result = gsk_render_node_deserialize (bytes, NULL, NULL);
+  g_bytes_unref (bytes);
+
+  return result;
+}
+
 typedef enum
 {
   KEEP_BOUNDS = (1 << 0),
@@ -642,6 +768,20 @@ static const TestSetup test_setups[] = {
     .create_test = colorflip_create_test,
     .create_reference = colorflip_create_reference,
   },
+  {
+    .name = "opaque",
+    .description = "Check the opaque rect",
+    .setup = opaque_setup,
+    .free = g_free,
+    .create_test = opaque_create_test,
+    .create_reference = opaque_create_reference,
+  },
+  {
+    .name = "serialize",
+    .description = "Serialize and deserialize",
+    .create_test = serialize_create_test,
+    .create_reference = NULL,
+  },
 };
 
 static void
@@ -665,26 +805,30 @@ run_single_test (const TestSetup *setup,
     render_bounds = NULL;
 
   if (setup->setup)
-    test_data = setup->setup (org_test);
+    {
+      test_data = setup->setup (org_test);
+
+      if (test_data == NULL && g_test_failed ())
+        {
+          /* Allow tests to fail/skip when setting up if
+           * they return NULL */
+          return;
+        }
+    }
   else
     test_data = NULL;
 
   if (setup->create_test)
     {
       test = setup->create_test (org_test, test_data);
-      save_node (test, file_name, setup->name, ".node");
     }
   else
     test = gsk_render_node_ref (org_test);
 
   rendered = gsk_renderer_render_texture (renderer, test, render_bounds);
-  save_image (rendered, file_name, setup->name, ".out.png");
 
   if (setup->create_reference)
-    {
-      reference = setup->create_reference (renderer, org_reference, test_data);
-      save_image (reference, file_name, setup->name, ".ref.png");
-    }
+    reference = setup->create_reference (renderer, org_reference, test_data);
   else
     reference = g_object_ref (org_reference);
 
@@ -694,10 +838,18 @@ run_single_test (const TestSetup *setup,
   diff = reftest_compare_textures (reference, rendered);
   if (diff)
     {
-      save_image (diff, file_name, setup->name, ".diff.png");
       g_test_fail ();
     }
 
+  if (diff || g_test_verbose ())
+    {
+      save_node (test, file_name, setup->name, ".node");
+      save_image (reference, file_name, setup->name, ".ref.png");
+      save_image (rendered, file_name, setup->name, ".out.png");
+      if (diff)
+        save_image (diff, file_name, setup->name, ".diff.png");
+    }
+
   g_clear_object (&diff);
   g_object_unref (rendered);
   g_object_unref (reference);
@@ -788,6 +940,8 @@ main (int argc, char **argv)
     { test_setups[5].name, 0, 0, G_OPTION_ARG_NONE, &test_enabled[5], test_setups[5].description, NULL },
     { test_setups[6].name, 0, 0, G_OPTION_ARG_NONE, &test_enabled[6], test_setups[6].description, NULL },
     { test_setups[7].name, 0, 0, G_OPTION_ARG_NONE, &test_enabled[7], test_setups[7].description, NULL },
+    { test_setups[8].name, 0, 0, G_OPTION_ARG_NONE, &test_enabled[8], test_setups[8].description, NULL },
+    { test_setups[9].name, 0, 0, G_OPTION_ARG_NONE, &test_enabled[9], test_setups[9].description, NULL },
     { NULL }
   };
   GOptionContext *context;
